@@ -1,6 +1,6 @@
 // info.h
 
-// Copyright (C) 2012-2013 by Werner Lemberg.
+// Copyright (C) 2012-2014 by Werner Lemberg.
 //
 // This file is part of the ttfautohint library, and may only be used,
 // modified, and distributed under the terms given in `COPYING'.  By
@@ -15,7 +15,6 @@
 #define __INFO_H__
 
 #include <ttfautohint.h>
-#include <numberset.h>
 
 extern "C" {
 
@@ -35,14 +34,19 @@ typedef struct Info_Data_
   bool dw_cleartype_strong_stem_width;
 
   int increase_x_height;
-  number_range* x_height_snapping_exceptions;
+  const char* x_height_snapping_exceptions_string;
+  int fallback_stem_width;
 
   bool windows_compatibility;
-  bool pre_hinting;
+  bool adjust_subglyphs;
   bool hint_composites;
+  char default_script[5];
   char fallback_script[5];
   bool symbol;
   bool dehint;
+  bool TTFA_info;
+
+  const char* control_name;
 } Info_Data;
 
 
