@@ -30,6 +30,7 @@
 #include <getopt.h>
 #include <limits.h>
 #include <unistd.h>
+#include <locale.h>
 
 #include <vector>
 #include <string>
@@ -131,6 +132,96 @@ progress(long curr_idx,
   return 0;
 }
 
+
+typedef struct Error_Data_
+{
+  const char* control_name;
+} Error_Data;
+
+
+void
+err(TA_Error error,
+    const char* error_string,
+    unsigned int errlinenum,
+    const char* errline,
+    const char* errpos,
+    void* user)
+{
+  Error_Data* data = static_cast<Error_Data*>(user);
+
+  if (!error)
+    return;
+
+  // We replace some terse error strings with more user-friendly versions.
+  if (error == TA_Err_Invalid_FreeType_Version)
+    fprintf(stderr,
+            "FreeType version 2.4.5 or higher is needed.\n"
+            "Perhaps using a wrong FreeType DLL?\n");
+  else if (error == TA_Err_Invalid_Font_Type)
+    fprintf(stderr,
+            "This font is not a valid font"
+              " in SFNT format with TrueType outlines.\n"
+            "In particular, CFF outlines are not supported.\n");
+  else if (error == TA_Err_Already_Processed)
+    fprintf(stderr,
+            "This font has already been processed with ttfautohint.\n");
+  else if (error == TA_Err_Missing_Legal_Permission)
+    fprintf(stderr,
+            "Bit 1 in the `fsType' field of the `OS/2' table is set:\n"
+            "This font must not be modified"
+              " without permission of the legal owner.\n"
+            "Use command line option `-i' to continue"
+              " if you have such a permission.\n");
+  else if (error == TA_Err_Missing_Unicode_CMap)
+    fprintf(stderr,
+            "No Unicode character map.\n");
+  else if (error == TA_Err_Missing_Symbol_CMap)
+    fprintf(stderr,
+            "No symbol character map.\n");
+  else if (error == TA_Err_Missing_Glyph)
+    fprintf(stderr,
+            "No glyph for a standard character"
+              " to derive standard width and height.\n"
+            "Please check the documentation for a list of"
+              " script-specific standard characters,\n"
+            "or use option `--symbol'.\n");
+  else
+  {
+    if (error < 0x100)
+      fprintf(stderr, "An error with code 0x%02x occurred"
+                        " while autohinting fonts",
+                      error);
+    else if (error >= 0x100 && error < 0x200)
+    {
+      fprintf(stderr, "An error with code 0x%03x occurred"
+                        " while parsing the argument of option `-X'",
+                      error);
+      fprintf(stderr, errline ? ":\n" : ".\n");
+
+      if (errline)
+        fprintf(stderr, "  %s\n", errline);
+      if (errpos && errline)
+        fprintf(stderr, "  %*s\n", int(errpos - errline + 1), "^");
+    }
+    else if (error >= 0x200 && error < 0x300)
+    {
+      fprintf(stderr, "%s:", data->control_name);
+      if (errlinenum)
+        fprintf(stderr, "%d:", errlinenum);
+      if (errpos && errline)
+        fprintf(stderr, "%d:", int(errpos - errline + 1));
+      if (error_string)
+        fprintf(stderr, " %s", error_string);
+      fprintf(stderr, " (0x%02X)\n", error);
+      if (errline)
+        fprintf(stderr, "  %s\n", errline);
+      if (errpos && errline)
+        fprintf(stderr, "  %*s\n", int(errpos - errline + 1), "^");
+    }
+  }
+}
+
+
 } // extern "C"
 #endif // !BUILD_GUI
 
@@ -187,20 +278,27 @@ show_help(bool
 "  -G, --hinting-limit=N      switch off hinting above this PPEM value\n"
 "                             (default: %d); value 0 means no limit\n"
 "  -h, --help                 display this help and exit\n"
+"  -H, --fallback-stem-width=N\n"
+"                             set fallback stem width\n"
+"                             (default: 50 font units at 2048 UPEM)\n"
 #ifdef BUILD_GUI
 "      --help-all             show Qt and X11 specific options also\n"
 #endif
 "  -i, --ignore-restrictions  override font license restrictions\n"
 "  -l, --hinting-range-min=N  the minimum PPEM value for hint sets\n"
 "                             (default: %d)\n"
+#ifndef BUILD_GUI
+"  -m, --control-file=FILE    get control instructions from FILE\n"
+#endif
 "  -n, --no-info              don't add ttfautohint info\n"
 "                             to the version string(s) in the `name' table\n"
-"  -p, --pre-hinting          apply original hints in advance\n",
+"  -p, --adjust-subglyphs     handle subglyph adjustments in exotic fonts\n",
           TA_HINTING_LIMIT, TA_HINTING_RANGE_MIN);
   fprintf(handle,
 "  -r, --hinting-range-max=N  the maximum PPEM value for hint sets\n"
 "                             (default: %d)\n"
 "  -s, --symbol               input is symbol font\n"
+"  -t, --ttfa-table           add TTFA information table\n"
 "  -v, --verbose              show progress information\n"
 "  -V, --version              print version information and exit\n"
 "  -w, --strong-stem-width=S  use strong stem width routine for modes S,\n"
@@ -283,10 +381,11 @@ show_help(bool
 "\n");
   fprintf(handle,
 "A hint set contains the optimal hinting for a certain PPEM value;\n"
-"the larger the hint set range, the more hint sets get computed,\n"
-"usually increasing the output font size.  Note, however,\n"
-"that the `gasp' table of the output file enables grayscale hinting\n"
+"the larger the hint set range (as given by options -l and -r),\n"
+"the more hint sets get computed, usually increasing the output font size.\n"
+"The `gasp' table of the output file always enables grayscale hinting\n"
 "for all sizes (limited by option -G, which is handled in the bytecode).\n"
+"Increasing the value of -G does not increase the output font size.\n"
 "\n");
   fprintf(handle,
 "Options -f and -D take a four-letter string that identifies a script.\n"
@@ -309,6 +408,27 @@ show_help(bool
     }
   }
   fprintf(handle,
+#ifndef BUILD_GUI
+"\n"
+"A control instructions file contains entries of the form\n"
+"\n"
+"  [<subfont idx>] <glyph id> l|r <points> [(<left offset>,<right offset>)]\n"
+"or\n"
+"  [<subfont idx>] <glyph id> n <points>\n"
+"or\n"
+"  [<subfont idx>] <glyph id> p <points> [x <shift>] [y <shift>] @ <ppems>\n"
+"\n"
+"`l' (`r') creates one-point segments with direction left (right).\n"
+"<left offset> and <right offset> specify offsets (in font units)\n"
+"relative to the corresponding points to give the segments a length.\n"
+"`n' removes points from horizontal segments, making them `weak' points.\n"
+"`p' applies delta exceptions to the specified points.\n"
+"\n"
+"<glyph id> is a glyph name or index, <shift> is a real number in px,\n"
+"<points> and <ppems> are integer ranges as with option `-X'.\n"
+"`#' starts a line comment, which gets ignored.\n"
+"Empty lines are ignored, too.\n"
+#endif
 "\n"
 #ifdef BUILD_GUI
 "A command-line version of this program is called `ttfautohint'.\n"
@@ -353,10 +473,13 @@ main(int argc,
   int hinting_range_max = 0;
   int hinting_limit = 0;
   int increase_x_height = 0;
+  int fallback_stem_width = 0;
+
   bool have_hinting_range_min = false;
   bool have_hinting_range_max = false;
   bool have_hinting_limit = false;
   bool have_increase_x_height = false;
+  bool have_fallback_stem_width = false;
 
   bool gray_strong_stem_width = false;
   bool gdi_cleartype_strong_stem_width = true;
@@ -364,9 +487,10 @@ main(int argc,
 
   bool ignore_restrictions = false;
   bool windows_compatibility = false;
-  bool pre_hinting = false;
+  bool adjust_subglyphs = false;
   bool hint_composites = false;
   bool no_info = false;
+  bool TTFA_info = false;
   bool symbol = false;
 
   const char* default_script = NULL;
@@ -382,9 +506,17 @@ main(int argc,
   bool debug = false;
 
   TA_Progress_Func progress_func = NULL;
+  TA_Error_Func err_func = err;
   TA_Info_Func info_func = info;
+
+  const char* control_name = NULL;
 #endif
 
+  // For real numbers (both parsing and displaying) we only use `.' as the
+  // decimal separator; similarly, we don't want localized formats like a
+  // thousands separator for any number.
+  setlocale(LC_NUMERIC, "C");
+
   // make GNU, Qt, and X11 command line options look the same;
   // we allow `--foo=bar', `--foo bar', `-foo=bar', `-foo bar',
   // and short options specific to ttfautohint
@@ -411,13 +543,16 @@ main(int argc,
 #endif
 
       // ttfautohint options
+      {"adjust-subglyphs", no_argument, NULL, 'p'},
       {"composites", no_argument, NULL, 'c'},
 #ifndef BUILD_GUI
+      {"control-file", required_argument, NULL, 'm'},
       {"debug", no_argument, NULL, DEBUG_OPTION},
 #endif
       {"default-script", required_argument, NULL, 'D'},
       {"dehint", no_argument, NULL, 'd'},
       {"fallback-script", required_argument, NULL, 'f'},
+      {"fallback-stem-width", required_argument, NULL, 'H'},
       {"hinting-limit", required_argument, NULL, 'G'},
       {"hinting-range-max", required_argument, NULL, 'r'},
       {"hinting-range-min", required_argument, NULL, 'l'},
@@ -427,6 +562,7 @@ main(int argc,
       {"pre-hinting", no_argument, NULL, 'p'},
       {"strong-stem-width", required_argument, NULL, 'w'},
       {"symbol", no_argument, NULL, 's'},
+      {"ttfa-table", no_argument, NULL, 't'},
       {"verbose", no_argument, NULL, 'v'},
       {"version", no_argument, NULL, 'V'},
       {"windows-compatibility", no_argument, NULL, 'W'},
@@ -462,7 +598,12 @@ main(int argc,
     };
 
     int option_index;
-    int c = getopt_long_only(argc, argv, "cdD:f:G:hil:npr:stVvw:Wx:X:",
+    int c = getopt_long_only(argc, argv,
+#ifdef BUILD_GUI
+                             "cdD:f:G:hH:il:npr:stVvw:Wx:X:",
+#else
+                             "cdD:f:G:hH:il:m:npr:stVvw:Wx:X:",
+#endif
                              long_options, &option_index);
     if (c == -1)
       break;
@@ -498,6 +639,11 @@ main(int argc,
 #endif
       break;
 
+    case 'H':
+      fallback_stem_width = atoi(optarg);
+      have_fallback_stem_width = true;
+      break;
+
     case 'i':
       ignore_restrictions = true;
       break;
@@ -507,12 +653,18 @@ main(int argc,
       have_hinting_range_min = true;
       break;
 
+#ifndef BUILD_GUI
+    case 'm':
+      control_name = optarg;
+      break;
+#endif
+
     case 'n':
       no_info = true;
       break;
 
     case 'p':
-      pre_hinting = true;
+      adjust_subglyphs = true;
       break;
 
     case 'r':
@@ -524,6 +676,10 @@ main(int argc,
       symbol = true;
       break;
 
+    case 't':
+      TTFA_info = true;
+      break;
+
     case 'v':
 #ifndef BUILD_GUI
       progress_func = progress;
@@ -593,8 +749,9 @@ main(int argc,
     // -d makes ttfautohint ignore all other hinting options
     have_default_script = false;
     have_fallback_script = false;
-    have_hinting_range_min = false;
+    have_fallback_stem_width = false;
     have_hinting_range_max = false;
+    have_hinting_range_min = false;
     have_hinting_limit = false;
     have_increase_x_height = false;
     have_x_height_snapping_exceptions_string = false;
@@ -614,6 +771,8 @@ main(int argc,
     increase_x_height = TA_INCREASE_X_HEIGHT;
   if (!have_x_height_snapping_exceptions_string)
     x_height_snapping_exceptions_string = "";
+  if (!have_fallback_stem_width)
+    fallback_stem_width = 0; /* redundant, but avoids a compiler warning */
 
 #ifndef BUILD_GUI
 
@@ -645,42 +804,11 @@ main(int argc,
                     " must be larger than or equal to 6\n");
     exit(EXIT_FAILURE);
   }
-
-  number_range* x_height_snapping_exceptions = NULL;
-
-  if (have_x_height_snapping_exceptions_string)
+  if (have_fallback_stem_width && fallback_stem_width <= 0)
   {
-    const char* s;
-
-
-    s = number_set_parse(x_height_snapping_exceptions_string,
-                         &x_height_snapping_exceptions,
-                         6, 0x7FFF);
-    if (*s)
-    {
-      if (x_height_snapping_exceptions == NUMBERSET_ALLOCATION_ERROR)
-        fprintf(stderr, "Allocation error while scanning"
-                        " x height snapping exceptions\n");
-      else {
-        if (x_height_snapping_exceptions == NUMBERSET_INVALID_CHARACTER)
-          fprintf(stderr, "Invalid character");
-        else if (x_height_snapping_exceptions == NUMBERSET_OVERFLOW)
-          fprintf(stderr, "Overflow");
-        else if (x_height_snapping_exceptions == NUMBERSET_INVALID_RANGE)
-          fprintf(stderr, "Invalid range");
-        else if (x_height_snapping_exceptions == NUMBERSET_OVERLAPPING_RANGES)
-          fprintf(stderr, "Overlapping ranges");
-        else if (x_height_snapping_exceptions == NUMBERSET_NOT_ASCENDING)
-          fprintf(stderr, "Values und ranges must be ascending");
-        fprintf(stderr, " in x height snapping exceptions:\n"
-                        "  \"%s\"\n"
-                        "   %*s\n",
-                        x_height_snapping_exceptions_string,
-                        int(s - x_height_snapping_exceptions_string + 1),
-                        "^");
-      }
-      exit(EXIT_FAILURE);
-    }
+    fprintf(stderr, "The fallback stem width"
+                    " must be a positive integer\n");
+    exit(EXIT_FAILURE);
   }
 
   if (have_default_script)
@@ -711,6 +839,13 @@ main(int argc,
     }
   }
 
+  if (symbol
+      && have_fallback_stem_width
+      && !strcmp(fallback_script, "none"))
+    fprintf(stderr,
+            "Warning: Setting a fallback stem width for a symbol font\n"
+            "         without setting a fallback script has no effect\n");
+
   int num_args = argc - optind;
 
   if (num_args > 2)
@@ -722,10 +857,11 @@ main(int argc,
     in = fopen(argv[optind], "rb");
     if (!in)
     {
-      fprintf(stderr, "The following error occurred while opening font `%s':\n"
-                      "\n"
-                      "  %s\n",
-                      argv[optind], strerror(errno));
+      fprintf(stderr,
+              "The following error occurred while opening font `%s':\n"
+              "\n"
+              "  %s\n",
+              argv[optind], strerror(errno));
       exit(EXIT_FAILURE);
     }
   }
@@ -748,10 +884,11 @@ main(int argc,
     out = fopen(argv[optind + 1], "wb");
     if (!out)
     {
-      fprintf(stderr, "The following error occurred while opening font `%s':\n"
-                      "\n"
-                      "  %s\n",
-                      argv[optind + 1], strerror(errno));
+      fprintf(stderr,
+              "The following error occurred while opening font `%s':\n"
+              "\n"
+              "  %s\n",
+              argv[optind + 1], strerror(errno));
       exit(EXIT_FAILURE);
     }
   }
@@ -762,8 +899,25 @@ main(int argc,
     out = stdout;
   }
 
-  const unsigned char* error_string;
+  FILE* control = NULL;
+  if (control_name)
+  {
+    control = fopen(control_name, "r");
+    if (!control)
+    {
+      fprintf(stderr,
+              "The following error occurred while open control file `%s':\n"
+              "\n"
+              "  %s\n",
+              control_name, strerror(errno));
+      exit(EXIT_FAILURE);
+    }
+  }
+  else
+    control = NULL;
+
   Progress_Data progress_data = {-1, 1, 0};
+  Error_Data error_data = {control_name};
   Info_Data info_data;
 
   if (no_info)
@@ -775,6 +929,8 @@ main(int argc,
     info_data.data_len = 0;
     info_data.data_wide_len = 0;
 
+    info_data.control_name = control_name;
+
     info_data.hinting_range_min = hinting_range_min;
     info_data.hinting_range_max = hinting_range_max;
     info_data.hinting_limit = hinting_limit;
@@ -784,11 +940,13 @@ main(int argc,
     info_data.dw_cleartype_strong_stem_width = dw_cleartype_strong_stem_width;
 
     info_data.windows_compatibility = windows_compatibility;
-    info_data.pre_hinting = pre_hinting;
+    info_data.adjust_subglyphs = adjust_subglyphs;
     info_data.hint_composites = hint_composites;
     info_data.increase_x_height = increase_x_height;
-    info_data.x_height_snapping_exceptions = x_height_snapping_exceptions;
+    info_data.x_height_snapping_exceptions_string = x_height_snapping_exceptions_string;
+    info_data.fallback_stem_width = fallback_stem_width;
     info_data.symbol = symbol;
+    info_data.TTFA_info = TTFA_info;
 
     strncpy(info_data.default_script,
             default_script,
@@ -814,30 +972,30 @@ main(int argc,
     SET_BINARY(stdout);
 
   TA_Error error =
-    TTF_autohint("in-file, out-file,"
+    TTF_autohint("in-file, out-file, control-file,"
                  "hinting-range-min, hinting-range-max, hinting-limit,"
                  "gray-strong-stem-width, gdi-cleartype-strong-stem-width,"
                  "dw-cleartype-strong-stem-width,"
-                 "error-string,"
                  "progress-callback, progress-callback-data,"
+                 "error-callback, error-callback-data,"
                  "info-callback, info-callback-data,"
                  "ignore-restrictions, windows-compatibility,"
-                 "pre-hinting, hint-composites,"
+                 "adjust-subglyphs, hint-composites,"
                  "increase-x-height, x-height-snapping-exceptions,"
-                 "default-script, fallback-script, symbol,"
-                 "dehint, debug",
-                 in, out,
+                 "fallback-stem-width, default-script, fallback-script,"
+                 "symbol, dehint, debug, TTFA-info",
+                 in, out, control,
                  hinting_range_min, hinting_range_max, hinting_limit,
                  gray_strong_stem_width, gdi_cleartype_strong_stem_width,
                  dw_cleartype_strong_stem_width,
-                 &error_string,
                  progress_func, &progress_data,
+                 err_func, &error_data,
                  info_func, &info_data,
                  ignore_restrictions, windows_compatibility,
-                 pre_hinting, hint_composites,
+                 adjust_subglyphs, hint_composites,
                  increase_x_height, x_height_snapping_exceptions_string,
-                 default_script, fallback_script, symbol,
-                 dehint, debug);
+                 fallback_stem_width, default_script, fallback_script,
+                 symbol, dehint, debug, TTFA_info);
 
   if (!no_info)
   {
@@ -845,55 +1003,14 @@ main(int argc,
     free(info_data.data_wide);
   }
 
-  number_set_free(x_height_snapping_exceptions);
-
-  if (error)
-  {
-    if (error == TA_Err_Invalid_FreeType_Version)
-      fprintf(stderr,
-              "FreeType version 2.4.5 or higher is needed.\n"
-              "Perhaps using a wrong FreeType DLL?\n");
-    else if (error == TA_Err_Invalid_Font_Type)
-      fprintf(stderr,
-              "This font is not a valid font"
-                " in SFNT format with TrueType outlines.\n"
-              "In particular, CFF outlines are not supported.\n");
-    else if (error == TA_Err_Already_Processed)
-      fprintf(stderr,
-              "This font has already been processed with ttfautohint.\n");
-    else if (error == TA_Err_Missing_Legal_Permission)
-      fprintf(stderr,
-              "Bit 1 in the `fsType' field of the `OS/2' table is set:\n"
-              "This font must not be modified"
-                " without permission of the legal owner.\n"
-              "Use command line option `-i' to continue"
-                " if you have such a permission.\n");
-    else if (error == TA_Err_Missing_Unicode_CMap)
-      fprintf(stderr,
-              "No Unicode character map.\n");
-    else if (error == TA_Err_Missing_Symbol_CMap)
-      fprintf(stderr,
-              "No symbol character map.\n");
-    else if (error == TA_Err_Missing_Glyph)
-      fprintf(stderr,
-              "No glyph for a standard character"
-              " to derive standard width and height.\n"
-              "Please check the documentation for a list of"
-              " script-specific standard characters,\n"
-              "or use option `--symbol'.\n");
-    else
-      fprintf(stderr,
-              "Error code `0x%02x' while autohinting font:\n"
-              "  %s\n", error, error_string);
-    exit(EXIT_FAILURE);
-  }
-
   if (in != stdin)
     fclose(in);
   if (out != stdout)
     fclose(out);
+  if (control)
+    fclose(control);
 
-  exit(EXIT_SUCCESS);
+  exit(error ? EXIT_FAILURE : EXIT_SUCCESS);
 
   return 0; // never reached
 
@@ -939,10 +1056,10 @@ main(int argc,
                    hinting_range_min, hinting_range_max, hinting_limit,
                    gray_strong_stem_width, gdi_cleartype_strong_stem_width,
                    dw_cleartype_strong_stem_width, increase_x_height,
-                   x_height_snapping_exceptions_string,
-                   ignore_restrictions, windows_compatibility, pre_hinting,
+                   x_height_snapping_exceptions_string, fallback_stem_width,
+                   ignore_restrictions, windows_compatibility, adjust_subglyphs,
                    hint_composites, no_info, default_script, fallback_script,
-                   symbol, dehint);
+                   symbol, dehint, TTFA_info);
 
     dummy.move(-50000, -50000);
     dummy.show();
@@ -958,10 +1075,10 @@ main(int argc,
                hinting_range_min, hinting_range_max, hinting_limit,
                gray_strong_stem_width, gdi_cleartype_strong_stem_width,
                dw_cleartype_strong_stem_width, increase_x_height,
-               x_height_snapping_exceptions_string,
-               ignore_restrictions, windows_compatibility, pre_hinting,
+               x_height_snapping_exceptions_string, fallback_stem_width,
+               ignore_restrictions, windows_compatibility, adjust_subglyphs,
                hint_composites, no_info, default_script, fallback_script,
-               symbol, dehint);
+               symbol, dehint, TTFA_info);
   gui.show();
 
   return app.exec();
