@@ -13,15 +13,111 @@
  */
 
 
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <limits.h>
 
+#include <sds.h>
 #include <numberset.h>
 
 
+number_range*
+number_set_new(int start,
+               int end,
+               int min,
+               int max)
+{
+  number_range* nr;
+
+
+  if (start > end)
+  {
+    int tmp;
+
+
+    tmp = start;
+    start = end;
+    end = tmp;
+  }
+
+  if (start < min || end > max)
+    return NUMBERSET_INVALID_RANGE;
+
+  nr = (number_range*)malloc(sizeof (number_range));
+  if (!nr)
+    return NUMBERSET_ALLOCATION_ERROR;
+
+  nr->start = start;
+  nr->end = end;
+  nr->next = NULL;
+
+  return nr;
+}
+
+
+number_range*
+number_set_prepend(number_range* list,
+                   number_range* element)
+{
+  if (!element)
+    return list;
+
+  if (!list)
+    return element;
+
+  if (element->start <= list->end)
+  {
+    if (element->end < list->start)
+      return NUMBERSET_NOT_ASCENDING;
+    else
+      return NUMBERSET_OVERLAPPING_RANGES;
+  }
+
+  if (list
+      && element->start == list->end + 1)
+  {
+    /* merge adjacent ranges */
+    list->end = element->end;
+
+    free(element);
+
+    return list;
+  }
+
+  element->next = list;
+
+  return element;
+}
+
+
+number_range*
+number_set_reverse(number_range* list)
+{
+  number_range* cur;
+
+
+  cur = list;
+  list = NULL;
+
+  while (cur)
+  {
+    number_range* tmp;
+
+
+    tmp = cur;
+    cur = cur->next;
+    tmp->next = list;
+    list = tmp;
+  }
+
+  return list;
+}
+
+
 const char*
 number_set_parse(const char* s,
                  number_range** number_set,
@@ -192,13 +288,7 @@ number_set_parse(const char* s,
 
   if (error_code)
   {
-    /* deallocate data */
-    while (cur)
-    {
-      tmp = cur;
-      cur = cur->next;
-      free(tmp);
-    }
+    number_set_free(cur);
 
     s = last_pos;
     if (number_set)
@@ -249,16 +339,12 @@ number_set_show(number_range* number_set,
                 int min,
                 int max)
 {
-  char* s;
-  char* s_new;
-  size_t s_len;
-  size_t s_len_new;
+  sds s;
+  size_t len;
+  char* res;
 
   number_range* nr = number_set;
 
-  char tmp[256];
-  int tmp_len;
-  int t;
   const char *comma;
 
 
@@ -268,64 +354,55 @@ number_set_show(number_range* number_set,
     max = INT_MAX;
   if (min > max)
   {
+    int t;
+
+
     t = min;
     min = max;
     max = t;
   }
 
-  /* we return an empty string for an empty number set */
-  /* (this is, number_set == NULL or unsuitable `min' and `max' values) */
-  s = (char*)malloc(1);
-  if (!s)
-    return NULL;
-  *s = '\0';
-
-  s_len = 1;
+  s = sdsempty();
 
   while (nr)
   {
     if (nr->start > max)
-      return s;
+      goto Exit;
     if (nr->end < min)
       goto Again;
 
-    comma = (s_len == 1) ? "" : ", ";
+    comma = *s ? ", " : "";
 
-    if (nr->start <= min
-        && nr->end >= max)
-      tmp_len = sprintf(tmp, "-");
+    if (nr->start == nr->end)
+      s = sdscatprintf(s, "%s%i", comma, nr->start);
+    else if (nr->start <= min
+             && nr->end >= max)
+      s = sdscatprintf(s, "-");
     else if (nr->start <= min)
-      tmp_len = sprintf(tmp, "-%i",
-                             nr->end);
+      s = sdscatprintf(s, "-%i", nr->end);
     else if (nr->end >= max)
-      tmp_len = sprintf(tmp, "%s%i-",
-                             comma, nr->start);
+      s = sdscatprintf(s, "%s%i-", comma, nr->start);
     else
-    {
-      if (nr->start == nr->end)
-        tmp_len = sprintf(tmp, "%s%i",
-                               comma, nr->start);
-      else
-        tmp_len = sprintf(tmp, "%s%i-%i",
-                               comma, nr->start, nr->end);
-    }
-
-    s_len_new = s_len + tmp_len;
-    s_new = (char*)realloc(s, s_len_new);
-    if (!s_new)
-    {
-      free(s);
-      return NULL;
-    }
-    strcpy(s_new + s_len - 1, tmp);
-    s_len = s_len_new;
-    s = s_new;
+      s = sdscatprintf(s, "%s%i-%i", comma, nr->start, nr->end);
 
   Again:
     nr = nr->next;
   }
 
-  return s;
+Exit:
+  if (!s)
+    return NULL;
+
+  /* we return an empty string for an empty number set */
+  /* (this is, number_set == NULL or unsuitable `min' and `max' values) */
+  len = sdslen(s) + 1;
+  res = (char*)malloc(len);
+  if (res)
+    memcpy(res, s, len);
+
+  sdsfree(s);
+
+  return res;
 }
 
 
@@ -349,4 +426,36 @@ number_set_is_element(number_range* number_set,
   return 0;
 }
 
+
+int
+number_set_get_first(number_set_iter* iter_p)
+{
+  if (!iter_p || !iter_p->range)
+    return 0;
+
+  iter_p->val = iter_p->range->start;
+
+  return iter_p->val;
+}
+
+
+int
+number_set_get_next(number_set_iter* iter_p)
+{
+  if (!iter_p || !iter_p->range)
+    return 0;
+
+  iter_p->val++;
+
+  if (iter_p->val > iter_p->range->end)
+  {
+    iter_p->range = iter_p->range->next;
+
+    if (iter_p->range)
+      iter_p->val = iter_p->range->start;
+  }
+
+  return iter_p->val;
+}
+
 /* end of numberset.c */
