@@ -1,7 +1,7 @@
 /* numberset.h */
 
 /*
- * Copyright (C) 2012-2013 by Werner Lemberg.
+ * Copyright (C) 2012-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -22,7 +22,7 @@ extern "C" {
 
 /*
  * A structure defining an integer range, to be used as a linked list.  It
- * gets allocated by a successful call to `parse_number_set'.  Use
+ * gets allocated by a successful call to `number_set_parse'.  Use
  * `number_set_free' to deallocate it.
  */
 
@@ -35,6 +35,43 @@ typedef struct number_range_
 } number_range;
 
 
+/*
+ * Create and initialize a `number_range' object.  In case of an allocation
+ * error, return NUMBERSET_ALLOCATION_ERROR.  If either `start' or `end'
+ * exceeds the range [min;max], return NUMBERSET_INVALID_RANGE.
+ */
+
+number_range*
+number_set_new(int start,
+               int end,
+               int min,
+               int max);
+
+
+/*
+ * Prepend a single `number_range' object `element' to `list' of
+ * `number_range' objects, which might be NULL.  `list' is expected to be
+ * stored in reversed order; consequently, the range in `element' must be
+ * larger than the first element of `list', otherwise an error is returned.
+ * If possible, the ranges of `element' and the first element of `list' are
+ * merged, in which case `element' gets deallocated.
+ *
+ * If `element' is NULL, return `list.
+ */
+
+number_range*
+number_set_prepend(number_range* list,
+                   number_range* element);
+
+
+/*
+ * Reverse a list of `number_range' objects.
+ */
+
+number_range*
+number_set_reverse(number_range* list);
+
+
 /*
  * Parse a description in string `s' for a set of non-negative integers
  * within the limits given by the input parameters `min' and `max', and
@@ -132,6 +169,38 @@ int
 number_set_is_element(number_range* number_set,
                       int number);
 
+
+/*
+ * A structure used to iterate over a number set.
+ */
+
+typedef struct number_set_iter_
+{
+  number_range* range;
+  int val;
+} number_set_iter;
+
+
+/*
+ * Get first element of a number set.  `iter_p' must be initialized with the
+ * `number_range' structure to iterate over.  After the call, `iter_p' is
+ * ready to be used in a call to `number_set_get_next'.
+ */
+
+int
+number_set_get_first(number_set_iter* iter_p);
+
+
+/*
+ * Get next element of a number set, using `iter_p' from a previous call to
+ * `number_set_get_first' or `number_set_get_next'.  If `iter_p->range' is
+ * NULL after the call, there is no next element, and the return value is
+ * undefined.
+ */
+
+int
+number_set_get_next(number_set_iter* iter_p);
+
 #ifdef __cplusplus
 } /* extern "C" */
 #endif
