@@ -1,7 +1,7 @@
 /* ta.h */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -23,14 +23,15 @@
 #include FT_TRUETYPE_TABLES_H
 #include FT_TRUETYPE_TAGS_H
 
+#include <ttfautohint.h>
+#include <sds.h>
+#include <numberset.h>
+
 #include "taloader.h"
 #include "taglobal.h"
 #include "tadummy.h"
 #include "talatin.h"
 
-#include <ttfautohint.h>
-#include <numberset.h>
-
 
 #define TTFAUTOHINT_GLYPH ".ttfautohint"
 #define TTFAUTOHINT_GLYPH_FIRST_BYTE "\x0C" /* first byte is string length */
@@ -48,6 +49,9 @@
 #define BYTE4(x) ((x) & 0x000000FFUL);
 
 
+/* an SFNT tag for our information table */
+#define TTAG_TTFA FT_MAKE_TAG('T', 'T', 'F', 'A')
+
 /* the length of a dummy `DSIG' table */
 #define DSIG_LEN 8
 
@@ -128,9 +132,11 @@ typedef struct glyf_Data_
   FT_UShort num_glyphs;
   GLYPH* glyphs;
 
-  /* this index gives the `master' globals for a `glyf' table; */
+  /* this field gives the `master' globals for a `glyf' table; */
   /* see function `TA_sfnt_handle_coverage' */
   TA_FaceGlobals master_globals;
+  /* for coverage bookkeeping */
+  FT_Bool adjusted;
 
   /* if a `glyf' table gets used in more than one subfont, */
   /* so do `cvt', `fpgm', and `prep' tables: */
@@ -139,16 +145,16 @@ typedef struct glyf_Data_
   FT_ULong fpgm_idx;
   FT_ULong prep_idx;
 
-  /* scripts present in a font get a running number */
-  FT_UInt script_ids[TA_SCRIPT_MAX];
-  FT_UInt num_used_scripts;
+  /* styles present in a font get a running number */
+  FT_UInt style_ids[TA_STYLE_MAX];
+  FT_UInt num_used_styles;
 
-  /* we have separate CVT data for each script */
-  FT_UInt cvt_offsets[TA_SCRIPT_MAX];
-  FT_UInt cvt_horz_width_sizes[TA_SCRIPT_MAX];
-  FT_UInt cvt_vert_width_sizes[TA_SCRIPT_MAX];
-  FT_UInt cvt_blue_zone_sizes[TA_SCRIPT_MAX];
-  FT_UInt cvt_blue_adjustment_offsets[TA_SCRIPT_MAX];
+  /* we have separate CVT data for each style */
+  FT_UInt cvt_offsets[TA_STYLE_MAX];
+  FT_UInt cvt_horz_width_sizes[TA_STYLE_MAX];
+  FT_UInt cvt_vert_width_sizes[TA_STYLE_MAX];
+  FT_UInt cvt_blue_zone_sizes[TA_STYLE_MAX];
+  FT_UInt cvt_blue_adjustment_offsets[TA_STYLE_MAX];
 } glyf_Data;
 
 /* an SFNT table */
@@ -196,7 +202,9 @@ typedef struct SFNT_
   FT_UShort max_components;
 } SFNT;
 
-/* our font object */
+typedef struct Control_ Control;
+
+/* our font object; the `FONT' typedef is in `taloader.h' */
 struct FONT_
 {
   FT_Library lib;
@@ -207,6 +215,9 @@ struct FONT_
   FT_Byte* out_buf;
   size_t out_len;
 
+  char* control_buf;
+  size_t control_len;
+
   SFNT* sfnts;
   FT_Long num_sfnts;
 
@@ -218,6 +229,18 @@ struct FONT_
   /* we have a single `gasp' table for all subfonts */
   FT_ULong gasp_idx;
 
+  /* the control instructions */
+  Control* control;
+
+  /* two generic pointers into the control instructions tree */
+  void* control_data_head;
+  void* control_data_cur;
+
+  /* two fields for handling one-point segment directions */
+  /* of the current glyph */
+  void* control_segment_dirs_head;
+  void* control_segment_dirs_cur;
+
   TA_LoaderRec loader[1]; /* the interface to the autohinter */
 
   /* configuration options */
@@ -230,27 +253,39 @@ struct FONT_
   FT_UInt hinting_limit;
   FT_UInt increase_x_height;
   number_range* x_height_snapping_exceptions;
+  FT_UInt fallback_stem_width;
   FT_Bool gray_strong_stem_width;
   FT_Bool gdi_cleartype_strong_stem_width;
   FT_Bool dw_cleartype_strong_stem_width;
   FT_Bool windows_compatibility;
-  FT_Bool pre_hinting;
+  FT_Bool adjust_subglyphs;
   FT_Bool hint_composites;
   FT_Bool ignore_restrictions;
-  TA_Script fallback_script;
+  TA_Style fallback_style;
+  TA_Script default_script;
   FT_Bool symbol;
   FT_Bool dehint;
   FT_Bool debug;
+  FT_Bool TTFA_info;
 };
 
 
 #include "tatables.h"
 #include "tabytecode.h"
+#include "tacontrol.h"
+
+
+/* in file `tascript.c' */
+extern const char* script_names[];
 
 
 const char*
 TA_get_error_message(FT_Error error);
 
+char*
+TA_font_dump_parameters(FONT* font,
+                        FT_Bool format);
+
 void
 TA_get_current_time(FT_ULong* high,
                     FT_ULong* low);
@@ -267,7 +302,8 @@ TA_font_init(FONT* font);
 void
 TA_font_unload(FONT* font,
                const char* in_buf,
-               char** out_bufp);
+               char** out_bufp,
+               const char* control_buf);
 
 FT_Error
 TA_font_file_read(FONT* font,
@@ -275,6 +311,9 @@ TA_font_file_read(FONT* font,
 FT_Error
 TA_font_file_write(FONT* font,
                    FILE* out_file);
+FT_Error
+TA_control_file_read(FONT* font,
+                     FILE* control_file);
 
 FT_Error
 TA_sfnt_build_glyph_instructions(SFNT* sfnt,
@@ -289,6 +328,11 @@ FT_Error
 TA_sfnt_build_cvt_table(SFNT* sfnt,
                         FONT* font);
 
+FT_Error
+TA_table_build_TTFA(FT_Byte** TTFA,
+                    FT_ULong* TTFA_len,
+                    FONT* font);
+
 FT_Error
 TA_table_build_DSIG(FT_Byte** DSIG);
 
@@ -312,9 +356,9 @@ TA_sfnt_create_glyf_data(SFNT* sfnt,
 FT_Error
 TA_sfnt_handle_coverage(SFNT* sfnt,
                         FONT* font);
-FT_Bool
-TA_sfnt_adjust_master_coverage(SFNT* sfnt,
-                               FONT* font);
+void
+TA_sfnt_adjust_coverage(SFNT* sfnt,
+                        FONT* font);
 #if 0
 void
 TA_sfnt_copy_master_coverage(SFNT* sfnt,
