@@ -13,12 +13,11 @@
  */
 
 
-#include <string.h>
+#include "ta.h"
+
 #include <stdbool.h> /* for llrb.h */
 
 #include "llrb.h" /* a red-black tree implementation */
-
-#include "ta.h"
 #include "tahints.h"
 
 
@@ -76,7 +75,7 @@ static int
 node2cmp(Node2* e1,
          Node2* e2)
 {
-  FT_Int  delta;
+  FT_Int delta;
 
 
   /* sort by edges ... */
@@ -92,7 +91,7 @@ static int
 node3cmp(Node3* e1,
          Node3* e2)
 {
-  FT_Int  delta;
+  FT_Int delta;
 
 
   /* sort by `before' edges ... */
@@ -245,6 +244,121 @@ TA_build_push(FT_Byte* bufp,
 }
 
 
+/*
+ * We optimize two common cases, replacing
+ *
+ *   NPUSHB A ... NPUSHB B [... NPUSHB C] ... CALL
+ *
+ * with
+ *
+ *   NPUSHB (A+B[+C]) ... CALL
+ *
+ * if possible
+ */
+
+FT_Byte*
+TA_optimize_push(FT_Byte* buf,
+                 FT_Byte** pos)
+{
+  FT_Byte sizes[3];
+  FT_Byte new_size1;
+  FT_Byte new_size2;
+
+  FT_UInt sum;
+  FT_UInt i;
+  FT_UInt pos_idx;
+
+  FT_Byte* p;
+  FT_Byte* bufp;
+
+
+  /* XXX improve handling of NPUSHW */
+  if (*(pos[0]) == NPUSHW
+      || *(pos[1]) == NPUSHW
+      || *(pos[2]) == NPUSHW)
+    return buf;
+
+  /* the point hints records block can be missing */
+  if (pos[0] == pos[1])
+  {
+    pos[1] = pos[2];
+    pos[2] = NULL;
+  }
+
+  /* there are at least two NPUSHB instructions */
+  /* (one of them directly at the start) */
+  sizes[0] = *(pos[0] + 1);
+  sizes[1] = *(pos[1] + 1);
+  sizes[2] = pos[2] ? *(pos[2] + 1) : 0;
+
+  sum = sizes[0] + sizes[1] + sizes[2];
+
+  if (sum > 2 * 0xFF)
+    return buf; /* nothing to do since we need three NPUSHB */
+  else if (!sizes[2] && (sum > 0xFF))
+    return buf; /* nothing to do since we need two NPUSHB */
+
+  if (sum > 0xFF)
+  {
+    /* reduce three NPUSHB to two */
+    new_size1 = 0xFF;
+    new_size2 = sum - 0xFF;
+  }
+  else
+  {
+    /* reduce two or three NPUSHB to one */
+    new_size1 = sum;
+    new_size2 = 0;
+  }
+
+  /* pack data */
+  p = buf;
+  bufp = buf;
+  pos_idx = 0;
+
+  if (new_size1 <= 8)
+    BCI(PUSHB_1 - 1 + new_size1);
+  else
+  {
+    BCI(NPUSHB);
+    BCI(new_size1);
+  }
+  for (i = 0; i < new_size1; i++)
+  {
+    if (p == pos[pos_idx])
+    {
+      pos_idx++;
+      p += 2; /* skip old NPUSHB */
+    }
+    *(bufp++) = *(p++);
+  }
+
+  if (new_size2)
+  {
+    if (new_size2 <= 8)
+      BCI(PUSHB_1 - 1 + new_size2);
+    else
+    {
+      BCI(NPUSHB);
+      BCI(new_size2);
+    }
+    for (i = 0; i < new_size2; i++)
+    {
+      if (p == pos[pos_idx])
+      {
+        pos_idx++;
+        p += 2;
+      }
+      *(bufp++) = *(p++);
+    }
+  }
+
+  BCI(CALL);
+
+  return bufp;
+}
+
+
 /* We add a subglyph for each composite glyph. */
 /* Since subglyphs must contain at least one point, */
 /* we have to adjust all point indices accordingly. */
@@ -485,6 +599,268 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
 }
 
 
+static void
+build_delta_exception(const Ctrl* ctrl,
+                      FT_UInt** delta_args,
+                      int* num_delta_args)
+{
+  int offset;
+  int ppem;
+  int x_shift;
+  int y_shift;
+
+
+  ppem = ctrl->ppem - CONTROL_DELTA_PPEM_MIN;
+
+  if (ppem < 16)
+    offset = 0;
+  else if (ppem < 32)
+    offset = 1;
+  else
+    offset = 2;
+
+  ppem -= offset << 4;
+
+  /*
+   * Using
+   *
+   *   delta_shift = 3   ,
+   *
+   * the possible shift values in the instructions are indexed as follows:
+   *
+   *    0   -1px
+   *    1   -7/8px
+   *   ...
+   *    7   -1/8px
+   *    8    1/8px
+   *   ...
+   *   14    7/8px
+   *   15    1px
+   *
+   * (note that there is no index for a zero shift).
+   */
+
+  if (ctrl->x_shift < 0)
+    x_shift = ctrl->x_shift + 8;
+  else
+    x_shift = ctrl->x_shift + 7;
+
+  if (ctrl->y_shift < 0)
+    y_shift = ctrl->y_shift + 8;
+  else
+    y_shift = ctrl->y_shift + 7;
+
+  /* add point index and exception specification to appropriate stack */
+  if (ctrl->x_shift)
+  {
+    *(delta_args[offset] + num_delta_args[offset]++) =
+      (ppem << 4) + x_shift;
+    *(delta_args[offset] + num_delta_args[offset]++) =
+      ctrl->point_idx;
+  }
+
+  if (ctrl->y_shift)
+  {
+    offset += 3;
+    *(delta_args[offset] + num_delta_args[offset]++) =
+      (ppem << 4) + y_shift;
+    *(delta_args[offset] + num_delta_args[offset]++) =
+      ctrl->point_idx;
+  }
+}
+
+
+static FT_Byte*
+TA_sfnt_build_delta_exceptions(SFNT* sfnt,
+                               FONT* font,
+                               FT_Long idx,
+                               FT_Byte* bufp)
+{
+  FT_Face face = font->loader->face;
+
+  int num_points;
+  int i;
+
+  FT_UShort num_stack_elements;
+
+  /* DELTAP[1-3] stacks for both x and y directions */
+  FT_UInt* delta_args[6] = {NULL, NULL, NULL, NULL, NULL, NULL};
+  int num_delta_args[6] = {0, 0, 0, 0, 0, 0};
+  FT_UInt* args = NULL;
+
+  FT_Bool need_words = 0;
+  FT_Bool need_word_counts = 0;
+  FT_Bool allocated = 0;
+
+
+  num_points = font->loader->gloader->base.outline.n_points;
+
+  /* loop over all fitting control instructions */
+  for (;;)
+  {
+    const Ctrl* ctrl = TA_control_get_ctrl(font);
+
+
+    if (!ctrl)
+      break;
+
+    /* check type */
+    if (!(ctrl->type == Control_Delta_before_IUP
+          || ctrl->type == Control_Delta_after_IUP))
+      break;
+
+    /* too large values of font and glyph indices in `ctrl' */
+    /* are handled by later calls of this function */
+    if (face->face_index < ctrl->font_idx
+        || idx < ctrl->glyph_idx)
+      break;
+
+    if (!allocated)
+    {
+      for (i = 0; i < 6; i++)
+      {
+        /* see the comment on allocating `ins_buf' in function */
+        /* `TA_sfnt_build_glyph_instructions' for more on the array sizes; */
+        /* we have to increase by 1 for the number of argument pairs */
+        delta_args[i] = (FT_UInt*)malloc((16 * 2 * num_points + 1)
+                                         * sizeof (FT_UInt));
+        if (!delta_args[i])
+        {
+          bufp = NULL;
+          goto Done;
+        }
+      }
+
+      allocated = 1;
+    }
+
+    /* since we walk sequentially over all glyphs (with points), */
+    /* and the control instruction entries have the same order, */
+    /* we don't need to test for equality of font and glyph indices: */
+    /* at this very point in the code we certainly have a hit */
+    build_delta_exception(ctrl, delta_args, num_delta_args);
+
+    if (ctrl->point_idx > 255)
+      need_words = 1;
+
+    TA_control_get_next(font);
+  }
+
+  /* nothing to do if no control instructions */
+  if (!allocated)
+    return bufp;
+
+  /* add number of argument pairs to the stacks */
+  for (i = 0; i < 6; i++)
+  {
+    if (num_delta_args[i])
+    {
+      int n = num_delta_args[i] >> 1;
+
+
+      if (n > 255)
+        need_word_counts = 1;
+
+      *(delta_args[i] + num_delta_args[i]) = n;
+      num_delta_args[i]++;
+    }
+  }
+
+  /* merge delta stacks into a single one */
+  if (need_words
+      || (!need_words && !need_word_counts))
+  {
+    FT_UInt num_args = 0;
+
+
+    for (i = 0; i < 6; i++)
+    {
+      FT_UInt* args_new;
+      FT_UInt num_args_new;
+
+
+      if (!num_delta_args[i])
+        continue;
+
+      num_args_new = num_args + num_delta_args[i];
+      args_new = (FT_UInt*)realloc(args, num_args_new * sizeof (FT_UInt));
+      if (!args_new)
+      {
+        bufp = NULL;
+        goto Done;
+      }
+
+      memcpy(args_new + num_args,
+             delta_args[i],
+             num_delta_args[i] * sizeof (FT_UInt));
+
+      args = args_new;
+      num_args = num_args_new;
+    }
+
+    num_stack_elements = num_args;
+
+    bufp = TA_build_push(bufp, args, num_args, need_words, 1);
+  }
+  else
+  {
+    num_stack_elements = 0;
+
+    /* stack elements are bytes, but counts need words */
+    for (i = 0; i < 6; i++)
+    {
+      int num_delta_arg;
+
+
+      if (!num_delta_args[i])
+        continue;
+
+      num_delta_arg = num_delta_args[i] - 1;
+
+      bufp = TA_build_push(bufp,
+                           delta_args[i],
+                           num_delta_arg,
+                           need_words,
+                           1);
+
+      num_stack_elements += num_delta_arg + 1;
+
+      num_delta_arg >>= 1;
+      BCI(PUSHW_1);
+      BCI(HIGH(num_delta_arg));
+      BCI(LOW(num_delta_arg));
+    }
+  }
+
+  /* emit the DELTA opcodes */
+  if (num_delta_args[5])
+    BCI(DELTAP3);
+  if (num_delta_args[4])
+    BCI(DELTAP2);
+  if (num_delta_args[3])
+    BCI(DELTAP1);
+
+  if (num_delta_args[2] || num_delta_args[1] || num_delta_args[0])
+    BCI(SVTCA_x);
+
+  if (num_delta_args[2])
+    BCI(DELTAP3);
+  if (num_delta_args[1])
+    BCI(DELTAP2);
+  if (num_delta_args[0])
+    BCI(DELTAP1);
+
+Done:
+  for (i = 0; i < 6; i++)
+    free(delta_args[i]);
+  free(args);
+
+  if (num_stack_elements > sfnt->max_stack_elements)
+    sfnt->max_stack_elements = num_stack_elements;
+  return bufp;
+}
+
+
 static FT_Byte*
 TA_sfnt_build_glyph_scaler(SFNT* sfnt,
                            Recorder* recorder,
@@ -502,6 +878,7 @@ TA_sfnt_build_glyph_scaler(SFNT* sfnt,
   FT_Bool need_words = 0;
   FT_Int p, q;
   FT_Int start, end;
+  FT_UShort num_storage;
   FT_UShort num_stack_elements;
 
 
@@ -568,6 +945,10 @@ TA_sfnt_build_glyph_scaler(SFNT* sfnt,
 
   BCI(CALL);
 
+  num_storage = sal_segment_offset;
+  if (num_storage > sfnt->max_storage)
+    sfnt->max_storage = num_storage;
+
   num_stack_elements = ADDITIONAL_STACK_ELEMENTS + num_args;
   if (num_stack_elements > sfnt->max_stack_elements)
     sfnt->max_stack_elements = num_stack_elements;
@@ -1867,7 +2248,7 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
   /* XXX: right now, we abuse this flag to control */
   /*      the global behaviour of the auto-hinter */
   load_flags = 1 << 29; /* vertical hinting only */
-  if (!font->pre_hinting)
+  if (!font->adjust_subglyphs)
   {
     if (font->hint_composites)
       load_flags |= FT_LOAD_NO_SCALE;
@@ -1885,6 +2266,13 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
   if (error)
     return error;
 
+  /* this data is needed for `ta_glyph_hints_reload' (in file `tahints.c') */
+  /* to modify `out' directions of points at the user's request */
+  /* (which will eventually become single-point segments) */
+  error = TA_control_segment_dir_collect(font, face->face_index, idx);
+  if (error)
+    return error;
+
 #ifdef TA_DEBUG
   /* temporarily disable some debugging output */
   /* to avoid getting the information twice */
@@ -1912,18 +2300,29 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
   if (!hints->num_points)
     return FT_Err_Ok;
 
-  /* we allocate a buffer which is certainly large enough */
-  /* to hold all of the created bytecode instructions; */
-  /* later on it gets reallocated to its real size */
-  ins_len = hints->num_points * 1000;
+  /*
+   * We allocate a buffer which is certainly large enough
+   * to hold all of the created bytecode instructions;
+   * later on it gets reallocated to its real size.
+   *
+   * The value `1000' is a very rough guess, not tested well.
+   *
+   * For delta exceptions, we have three DELTA commands,
+   * covering 3*16 ppem values.
+   * Since a point index can be larger than 255,
+   * we assume two bytes everywhere for the necessary PUSH calls.
+   * This value must be doubled for the other arguments of DELTA.
+   * Additionally, we have both x and y deltas,
+   * which need to be handled separately in the bytecode.
+   * In summary, this is approx. 3*16 * 2*2 * 2 = 400 bytes per point,
+   * adding some bytes for the necessary overhead.
+   */
+  ins_len = hints->num_points
+            * (1000 + ((font->control_data_head != NULL) ? 400 : 0));
   ins_buf = (FT_Byte*)malloc(ins_len);
   if (!ins_buf)
     return FT_Err_Out_Of_Memory;
 
-  /* initialize array with an invalid bytecode */
-  /* so that we can easily find the array length at reallocation time */
-  memset(ins_buf, INS_A0, ins_len);
-
   /* handle composite glyph */
   if (font->loader->gloader->base.num_subglyphs)
   {
@@ -2116,11 +2515,6 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
       goto Err;
     }
 
-    /* clear the rest of the temporarily used part of `ins_buf' */
-    p = bufp;
-    while (*p != INS_A0)
-      *(p++) = INS_A0;
-
     goto Done;
   }
 
@@ -2157,135 +2551,26 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
     goto Err;
   }
 
-  /* XXX improve handling of NPUSHW */
-  if (num_action_hints_records == 1
-      && *(pos[0]) != NPUSHW && *(pos[1]) != NPUSHW && *(pos[2]) != NPUSHW)
-  {
-    /*
-     * we optimize two common cases, replacing
-     *
-     *   NPUSHB A ... NPUSHB B [... NPUSHB C] ... CALL
-     *
-     * with
-     *
-     *   NPUSHB (A+B[+C]) ... CALL
-     *
-     * if possible
-     */
-    FT_Byte sizes[3];
-    FT_Byte new_size1;
-    FT_Byte new_size2;
-
-    FT_UInt sum;
-    FT_UInt i;
-    FT_UInt pos_idx;
-
-
-    /* the point hints records block can be missing */
-    if (pos[0] == pos[1])
-    {
-      pos[1] = pos[2];
-      pos[2] = NULL;
-    }
-
-    /* there are at least two NPUSHB instructions */
-    /* (one of them directly at the start) */
-    sizes[0] = *(pos[0] + 1);
-    sizes[1] = *(pos[1] + 1);
-    sizes[2] = pos[2] ? *(pos[2] + 1) : 0;
-
-    sum = sizes[0] + sizes[1] + sizes[2];
-
-    if (sum > 2 * 0xFF)
-      goto Done2; /* nothing to do since we need three NPUSHB */
-    else if (!sizes[2] && (sum > 0xFF))
-      goto Done2; /* nothing to do since we need two NPUSHB */
-
-    if (sum > 0xFF)
-    {
-      /* reduce three NPUSHB to two */
-      new_size1 = 0xFF;
-      new_size2 = sum - 0xFF;
-    }
-    else
-    {
-      /* reduce two or three NPUSHB to one */
-      new_size1 = sum;
-      new_size2 = 0;
-    }
-
-    /* pack data */
-    p = ins_buf;
-    bufp = ins_buf;
-    pos_idx = 0;
-
-    if (new_size1 <= 8)
-      BCI(PUSHB_1 - 1 + new_size1);
-    else
-    {
-      BCI(NPUSHB);
-      BCI(new_size1);
-    }
-    for (i = 0; i < new_size1; i++)
-    {
-      if (p == pos[pos_idx])
-      {
-        pos_idx++;
-        p += 2; /* skip old NPUSHB */
-      }
-      *(bufp++) = *(p++);
-    }
-
-    if (new_size2)
-    {
-      if (new_size2 <= 8)
-        BCI(PUSHB_1 - 1 + new_size2);
-      else
-      {
-        BCI(NPUSHB);
-        BCI(new_size2);
-      }
-      for (i = 0; i < new_size2; i++)
-      {
-        if (p == pos[pos_idx])
-        {
-          pos_idx++;
-          p += 2;
-        }
-        *(bufp++) = *(p++);
-      }
-    }
-
-    BCI(CALL);
-  }
-
-Done2:
-  /* clear the rest of the temporarily used part of `ins_buf' */
-  p = bufp;
-  while (*p != INS_A0)
-    *(p++) = INS_A0;
+  if (num_action_hints_records == 1)
+    bufp = TA_optimize_push(ins_buf, pos);
 
 Done:
   TA_free_hints_records(action_hints_records, num_action_hints_records);
   TA_free_hints_records(point_hints_records, num_point_hints_records);
   TA_free_recorder(&recorder);
 
-  /* we are done, so reallocate the instruction array to its real size */
-  if (*bufp == INS_A0)
+Done1:
+  /* handle delta exceptions */
+  if (font->control_data_head)
   {
-    /* search backwards */
-    while (*bufp == INS_A0)
-      bufp--;
-    bufp++;
-  }
-  else
-  {
-    /* search forwards */
-    while (*bufp != INS_A0)
-      bufp++;
+    bufp = TA_sfnt_build_delta_exceptions(sfnt, font, idx, bufp);
+    if (!bufp)
+    {
+      error = FT_Err_Out_Of_Memory;
+      goto Err;
+    }
   }
 
-Done1:
   ins_len = bufp - ins_buf;
 
   if (ins_len > sfnt->max_instructions)
