@@ -1,7 +1,7 @@
 /* tabytecode.h */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -310,7 +310,8 @@
 #define sal_ref sal_best + 1
 #define sal_func sal_ref + 1
 #define sal_anchor sal_func + 1
-#define sal_vwidth_data_offset sal_anchor + 1
+#define sal_stem_width_function sal_anchor + 1
+#define sal_vwidth_data_offset sal_stem_width_function + 1
 #define sal_scale sal_vwidth_data_offset + 1
 #define sal_point_min sal_scale + 1
 #define sal_point_max sal_point_min + 1
@@ -332,19 +333,20 @@
 #define bci_cvt_rescale bci_loop + 1
 #define bci_cvt_rescale_range bci_cvt_rescale + 1
 #define bci_vwidth_data_store bci_cvt_rescale_range + 1
-#define bci_blue_round bci_vwidth_data_store + 1
-#define bci_blue_round_range bci_blue_round + 1
+#define bci_smooth_blue_round bci_vwidth_data_store + 1
+#define bci_strong_blue_round bci_smooth_blue_round + 1
+#define bci_blue_round_range bci_strong_blue_round + 1
 #define bci_decrement_component_counter bci_blue_round_range + 1
 #define bci_get_point_extrema bci_decrement_component_counter + 1
 #define bci_nibbles bci_get_point_extrema + 1
 #define bci_number_set_is_element bci_nibbles + 1
 #define bci_number_set_is_element2 bci_number_set_is_element + 1
 
-/* 17 */
+/* 18 */
 #define bci_create_segment bci_number_set_is_element2 + 1
 #define bci_create_segments bci_create_segment + 1
 
-/* 19 */
+/* 20 */
 /* the next ten entries must stay in this order */
 #define bci_create_segments_0 bci_create_segments + 1
 #define bci_create_segments_1 bci_create_segments_0 + 1
@@ -359,7 +361,7 @@
 
 #define bci_create_segments_composite bci_create_segments_9 + 1
 
-/* 30 */
+/* 31 */
 /* the next ten entries must stay in this order */
 #define bci_create_segments_composite_0 bci_create_segments_composite + 1
 #define bci_create_segments_composite_1 bci_create_segments_composite_0 + 1
@@ -372,25 +374,25 @@
 #define bci_create_segments_composite_8 bci_create_segments_composite_7 + 1
 #define bci_create_segments_composite_9 bci_create_segments_composite_8 + 1
 
-/* 40 */
+/* 41 */
 #define bci_align_point bci_create_segments_composite_9 + 1
 #define bci_align_segment bci_align_point + 1
 #define bci_align_segments bci_align_segment + 1
 
-/* 43 */
+/* 44 */
 #define bci_scale_contour bci_align_segments + 1
 #define bci_scale_glyph bci_scale_contour + 1
 #define bci_scale_composite_glyph bci_scale_glyph + 1
 #define bci_shift_contour bci_scale_composite_glyph + 1
 #define bci_shift_subglyph bci_shift_contour + 1
 
-/* 48 */
+/* 49 */
 #define bci_ip_outer_align_point bci_shift_subglyph + 1
 #define bci_ip_on_align_points bci_ip_outer_align_point + 1
 #define bci_ip_between_align_point bci_ip_on_align_points + 1
 #define bci_ip_between_align_points bci_ip_between_align_point + 1
 
-/* 52 */
+/* 53 */
 #define bci_adjust_common bci_ip_between_align_points + 1
 #define bci_stem_common bci_adjust_common + 1
 #define bci_serif_common bci_stem_common + 1
@@ -398,12 +400,12 @@
 #define bci_serif_link1_common bci_serif_anchor_common + 1
 #define bci_serif_link2_common bci_serif_link1_common + 1
 
-/* 58 */
+/* 59 */
 #define bci_lower_bound bci_serif_link2_common + 1
 #define bci_upper_bound bci_lower_bound + 1
 #define bci_upper_lower_bound bci_upper_bound + 1
 
-/* 61 */
+/* 62 */
 #define bci_adjust_bound bci_upper_lower_bound + 1
 #define bci_stem_bound bci_adjust_bound + 1
 #define bci_link bci_stem_bound + 1
@@ -414,23 +416,23 @@
 /* the order of the `bci_action_*' entries must correspond */
 /* to the order of the TA_Action enumeration entries (in `tahints.h') */
 
-/* 67 */
+/* 68 */
 #define bci_action_ip_before bci_stem + 1
 #define bci_action_ip_after bci_action_ip_before + 1
 #define bci_action_ip_on bci_action_ip_after + 1
 #define bci_action_ip_between bci_action_ip_on + 1
 
-/* 71 */
+/* 72 */
 #define bci_action_blue bci_action_ip_between + 1
 #define bci_action_blue_anchor bci_action_blue + 1
 
-/* 73 */
+/* 74 */
 #define bci_action_anchor bci_action_blue_anchor + 1
 #define bci_action_anchor_serif bci_action_anchor + 1
 #define bci_action_anchor_round bci_action_anchor_serif + 1
 #define bci_action_anchor_round_serif bci_action_anchor_round + 1
 
-/* 77 */
+/* 78 */
 #define bci_action_adjust bci_action_anchor_round_serif + 1
 #define bci_action_adjust_serif bci_action_adjust + 1
 #define bci_action_adjust_round bci_action_adjust_serif + 1
@@ -440,13 +442,13 @@
 #define bci_action_adjust_bound_round bci_action_adjust_bound_serif + 1
 #define bci_action_adjust_bound_round_serif bci_action_adjust_bound_round + 1
 
-/* 85 */
+/* 86 */
 #define bci_action_link bci_action_adjust_bound_round_serif + 1
 #define bci_action_link_serif bci_action_link + 1
 #define bci_action_link_round bci_action_link_serif + 1
 #define bci_action_link_round_serif bci_action_link_round + 1
 
-/* 89 */
+/* 90 */
 #define bci_action_stem bci_action_link_round_serif + 1
 #define bci_action_stem_serif bci_action_stem + 1
 #define bci_action_stem_round bci_action_stem_serif + 1
@@ -456,31 +458,31 @@
 #define bci_action_stem_bound_round bci_action_stem_bound_serif + 1
 #define bci_action_stem_bound_round_serif bci_action_stem_bound_round + 1
 
-/* 97 */
+/* 98 */
 #define bci_action_serif bci_action_stem_bound_round_serif + 1
 #define bci_action_serif_lower_bound bci_action_serif + 1
 #define bci_action_serif_upper_bound bci_action_serif_lower_bound + 1
 #define bci_action_serif_upper_lower_bound bci_action_serif_upper_bound + 1
 
-/* 101 */
+/* 102 */
 #define bci_action_serif_anchor bci_action_serif_upper_lower_bound + 1
 #define bci_action_serif_anchor_lower_bound bci_action_serif_anchor + 1
 #define bci_action_serif_anchor_upper_bound bci_action_serif_anchor_lower_bound + 1
 #define bci_action_serif_anchor_upper_lower_bound bci_action_serif_anchor_upper_bound + 1
 
-/* 105 */
+/* 106 */
 #define bci_action_serif_link1 bci_action_serif_anchor_upper_lower_bound + 1
 #define bci_action_serif_link1_lower_bound bci_action_serif_link1 + 1
 #define bci_action_serif_link1_upper_bound bci_action_serif_link1_lower_bound + 1
 #define bci_action_serif_link1_upper_lower_bound bci_action_serif_link1_upper_bound + 1
 
-/* 109 */
+/* 110 */
 #define bci_action_serif_link2 bci_action_serif_link1_upper_lower_bound + 1
 #define bci_action_serif_link2_lower_bound bci_action_serif_link2 + 1
 #define bci_action_serif_link2_upper_bound bci_action_serif_link2_lower_bound + 1
 #define bci_action_serif_link2_upper_lower_bound bci_action_serif_link2_upper_bound + 1
 
-/* 113 */
+/* 114 */
 #define bci_hint_glyph bci_action_serif_link2_upper_lower_bound + 1
 
 #define NUM_FDEFS bci_hint_glyph + 1 /* must be last */
@@ -493,12 +495,10 @@
 /* (assigned in `prep' or `fpgm') */
 
 #define cvtl_temp 0 /* used for creating twilight points */
-#define cvtl_0x10000 cvtl_temp + 1
-#define cvtl_funits_to_pixels cvtl_0x10000 + 1
+#define cvtl_funits_to_pixels cvtl_temp + 1
 #define cvtl_is_subglyph cvtl_funits_to_pixels + 1
-#define cvtl_num_used_scripts cvtl_is_subglyph + 1
-#define cvtl_stem_width_function cvtl_num_used_scripts + 1
-#define cvtl_is_element cvtl_stem_width_function + 1
+#define cvtl_use_strong_functions cvtl_is_subglyph + 1
+#define cvtl_is_element cvtl_use_strong_functions + 1
 
 #define cvtl_max_runtime cvtl_is_element + 1 /* must be last */
 
@@ -506,9 +506,9 @@
 /* symbolic names for build-time CVT locations */
 /* (assigned in `cvt') */
 
-/* note that each script has its own set of CVT data, */
+/* note that each style has its own set of CVT data, */
 /* to be accessed using the offsets in the `cvt_offsets' array */
-/* first script index is 0 */
+/* first style index is 0 */
 
 /* the following macros access the variables `font' and `sfnt' */
 #define CVT_DATA ((glyf_Data*)(font->tables[sfnt->glyf_idx].data))
@@ -517,13 +517,13 @@
  * we have the following layout in the CVT table:
  *
  *   . values initialized at runtime (`cvtl_max_runtime' elements)
- *   . scaling values for each script ID (`num_used_script' elements)    (*)
- *   . offset to the vertical stem widths array for each script ID       (*)
- *     (`num_used_script' elements)
- *   . size of the vertical stem widths array for each script ID         (*)
- *     (`num_used_script' elements)
+ *   . scaling values for each style ID (`num_used_styles' elements)     (*)
+ *   . offset to the vertical stem widths array for each style ID        (*+)
+ *     (`num_used_styles' elements)
+ *   . size of the vertical stem widths array for each style ID          (*+)
+ *     (`num_used_styles' elements)
  *
- *   script ID 0:
+ *   style ID 0:
  *     . horizontal standard width (1 element)
  *     . horizontal stem widths (`cvt_horz_width_sizes[id_to_idx(0)]'
  *       elements)
@@ -532,65 +532,71 @@
  *       elements)
  *     . flat blue zones (`cvt_blue_zone_sizes[id_to_idx(0)]' elements)
  *     . round blue zones (`cvt_blue_zone_sizes[id_to_idx(0)]' elements)
- *   script ID 1:
+ *   style ID 1:
  *     ...
  *
  * (*) see function `bci_create_segments' how these three arrays get
  *     accessed
  *
+ * (+) the values are stored as multiples of 64 to avoid rounding issues
+ *     with non-square pixels (CVT values might be scaled forth and back by
+ *     the bytecode interpreter while storing and reading if the aspect
+ *     ratio between the horizontal and vertical resolution isn't equal
+ *     to 1)
+ *
  * note that the `id_to_idx' function is hypothetic since the code works
- * exactly the opposite way: the `cvt_*' arrays are indexed by the script
- * index, and the `script_ids' array maps script indices to script IDs
+ * exactly the opposite way: the `cvt_*' arrays are indexed by the style
+ * index, and the `style_ids' array maps style indices to style IDs
  */
 
-/* scaling value index of script ID id */
+/* scaling value index of style ID id */
 #define CVT_SCALING_VALUE_OFFSET(id) \
-          cvtl_max_runtime + (id)
+          (cvtl_max_runtime + (id))
 
-/* vwidth offset data of script ID id */
+/* vwidth offset data of style ID id */
 #define CVT_VWIDTH_OFFSET_DATA(id) \
-          CVT_SCALING_VALUE_OFFSET(id) \
-          + CVT_DATA->num_used_scripts \
+          (CVT_SCALING_VALUE_OFFSET(id) \
+           + CVT_DATA->num_used_styles) \
 
-/* vwidth size data of script ID id */
+/* vwidth size data of style ID id */
 #define CVT_VWIDTH_SIZE_DATA(id) \
-          CVT_VWIDTH_OFFSET_DATA(id) \
-          + CVT_DATA->num_used_scripts
+          (CVT_VWIDTH_OFFSET_DATA(id) \
+           + CVT_DATA->num_used_styles)
 
-/* horizontal standard width indices of script i */
+/* horizontal standard width indices of style i */
 #define CVT_HORZ_STANDARD_WIDTH_OFFSET(i) \
-          cvtl_max_runtime \
-          + 3 * CVT_DATA->num_used_scripts \
-          + CVT_DATA->cvt_offsets[i]
-/* start and size of horizontal stem widths array of script i */
+          (cvtl_max_runtime \
+           + 3 * CVT_DATA->num_used_styles \
+           + CVT_DATA->cvt_offsets[i])
+/* start and size of horizontal stem widths array of style i */
 #define CVT_HORZ_WIDTHS_OFFSET(i) \
-          CVT_HORZ_STANDARD_WIDTH_OFFSET(i) + 1
+          (CVT_HORZ_STANDARD_WIDTH_OFFSET(i) + 1)
 #define CVT_HORZ_WIDTHS_SIZE(i) \
-          CVT_DATA->cvt_horz_width_sizes[i]
+          (CVT_DATA->cvt_horz_width_sizes[i])
 
-/* vertical standard width indices of script i */
+/* vertical standard width indices of style i */
 #define CVT_VERT_STANDARD_WIDTH_OFFSET(i) \
-          CVT_HORZ_WIDTHS_OFFSET(i) + CVT_HORZ_WIDTHS_SIZE(i)
-/* start and size of vertical stem widths array of script i */
+          (CVT_HORZ_WIDTHS_OFFSET(i) + CVT_HORZ_WIDTHS_SIZE(i))
+/* start and size of vertical stem widths array of style i */
 #define CVT_VERT_WIDTHS_OFFSET(i) \
-          CVT_VERT_STANDARD_WIDTH_OFFSET(i) + 1
+          (CVT_VERT_STANDARD_WIDTH_OFFSET(i) + 1)
 #define CVT_VERT_WIDTHS_SIZE(i) \
-          CVT_DATA->cvt_vert_width_sizes[i]
+          (CVT_DATA->cvt_vert_width_sizes[i])
 
-/* number of blue zones (including artificial ones) of script i */
+/* number of blue zones (including artificial ones) of style i */
 #define CVT_BLUES_SIZE(i) \
-          CVT_DATA->cvt_blue_zone_sizes[i]
+          (CVT_DATA->cvt_blue_zone_sizes[i])
 
-/* start of blue zone arrays for flat and round edges of script i */
+/* start of blue zone arrays for flat and round edges of style i */
 #define CVT_BLUE_REFS_OFFSET(i) \
-          CVT_VERT_WIDTHS_OFFSET(i) + CVT_VERT_WIDTHS_SIZE(i)
+          (CVT_VERT_WIDTHS_OFFSET(i) + CVT_VERT_WIDTHS_SIZE(i))
 #define CVT_BLUE_SHOOTS_OFFSET(i) \
-          CVT_BLUE_REFS_OFFSET(i) + CVT_BLUES_SIZE(i)
+          (CVT_BLUE_REFS_OFFSET(i) + CVT_BLUES_SIZE(i))
 
-/* x height blue zone (shoot) index of script i (valid if < 0xFFFF) */
+/* x height blue zone (shoot) index of style i (valid if < 0xFFFF) */
 #define CVT_X_HEIGHT_BLUE_OFFSET(i) \
-          CVT_BLUE_SHOOTS_OFFSET(i) \
-          + CVT_DATA->cvt_blue_adjustment_offsets[i]
+          (CVT_BLUE_SHOOTS_OFFSET(i) \
+           + CVT_DATA->cvt_blue_adjustment_offsets[i])
 
 
 extern FT_Byte ttfautohint_glyph_bytecode[7];
