@@ -85,7 +85,15 @@ TA_sfnt_compute_global_hints(SFNT* sfnt,
     }
 
     if (!glyph_index)
+    {
+      /* in case of a symbol font, */
+      /* we only proceed if a fallback style is set */
+      if (font->symbol
+          && font->fallback_style != TA_STYLE_NONE_DFLT
+          && font->fallback_style == style_idx)
+        goto Symbol;
       return TA_Err_Missing_Glyph;
+    }
 
     /*
      * We now know that HarfBuzz can access the standard character in the
@@ -114,6 +122,7 @@ TA_sfnt_compute_global_hints(SFNT* sfnt,
     }
   }
 
+Symbol:
   load_flags = 1 << 29; /* vertical hinting only */
   error = ta_loader_load_glyph(font, face, glyph_index, load_flags);
 
@@ -246,6 +255,9 @@ TA_table_build_cvt(FT_Byte** cvt,
   /* loop again over all styles and copy CVT data */
   for (i = 0; i < TA_STYLE_MAX; i++)
   {
+    FT_UInt default_width = 50 * sfnt->face->units_per_EM / 2048;
+
+
     /* collect offsets */
     data->cvt_offsets[i] = ((FT_UInt)(bufp - buf) - cvt_offset) >> 1;
 
@@ -282,8 +294,8 @@ TA_table_build_cvt(FT_Byte** cvt,
     }
     else
     {
-      *(bufp++) = 0;
-      *(bufp++) = 50;
+      *(bufp++) = HIGH(default_width);
+      *(bufp++) = LOW(default_width);
     }
 
     for (j = 0; j < hwidth_count; j++)
@@ -302,8 +314,8 @@ TA_table_build_cvt(FT_Byte** cvt,
     }
     else
     {
-      *(bufp++) = 0;
-      *(bufp++) = 50;
+      *(bufp++) = HIGH(default_width);
+      *(bufp++) = LOW(default_width);
     }
 
     for (j = 0; j < vwidth_count; j++)
