@@ -70,4 +70,45 @@ TA_font_file_write(FONT* font,
   return TA_Err_Ok;
 }
 
+
+FT_Error
+TA_control_file_read(FONT* font,
+                     FILE* control_file)
+{
+  char* buf[BUF_SIZE];
+  size_t control_len = 0;
+  size_t read_bytes;
+
+
+  font->control_buf = (char*)malloc(BUF_SIZE);
+  if (!font->control_buf)
+    return FT_Err_Out_Of_Memory;
+
+  while ((read_bytes = fread(buf, 1, BUF_SIZE, control_file)) > 0)
+  {
+    char* control_buf_new;
+
+
+    /* we store the data as a C string, allocating one more byte */
+    control_buf_new = (char*)realloc(font->control_buf,
+                                     control_len + read_bytes + 1);
+    if (!control_buf_new)
+      return FT_Err_Out_Of_Memory;
+    else
+      font->control_buf = control_buf_new;
+
+    memcpy(font->control_buf + control_len, buf, read_bytes);
+
+    control_len += read_bytes;
+  }
+
+  if (ferror(control_file))
+    return FT_Err_Invalid_Stream_Read;
+
+  font->control_len = control_len;
+  font->control_buf[control_len] = '\0';
+
+  return TA_Err_Ok;
+}
+
 /* end of tafile.c */
