@@ -17,6 +17,8 @@
 
 /* heavily modified 2011 by Werner Lemberg <wl@gnu.org> */
 
+#include "ta.h"
+
 #include <string.h>
 #include <stdlib.h>
 #include "tahints.h"
@@ -69,7 +71,8 @@ Exit:
 }
 
 
-/* get new edge for given axis, direction, and position */
+/* get new edge for given axis, direction, and position, */
+/* without initializing the edge itself */
 
 FT_Error
 ta_axis_hints_new_edge(TA_AxisHints axis,
@@ -131,10 +134,6 @@ ta_axis_hints_new_edge(TA_AxisHints axis,
 
   axis->num_edges++;
 
-  memset(edge, 0, sizeof (TA_EdgeRec));
-  edge->fpos = (FT_Short)fpos;
-  edge->dir = (FT_Char)dir;
-
 Exit:
   *anedge = edge;
   return error;
@@ -204,11 +203,11 @@ ta_glyph_hints_dump_points(TA_GlyphHints hints)
 
   TA_LOG(("Table of points:\n"
           "  [ index |  xorg |  yorg | xscale | yscale"
-          " |  xfit |  yfit |  flags ]\n"));
+          " |  xfit |  yfit | flags ]\n"));
 
   for (point = points; point < limit; point++)
     TA_LOG(("  [ %5d | %5d | %5d | %6.2f | %6.2f"
-            " | %5.2f | %5.2f | %c%c%c%c%c%c ]\n",
+            " | %5.2f | %5.2f |   %c   ]\n",
             point - points,
             point->fx,
             point->fy,
@@ -216,12 +215,7 @@ ta_glyph_hints_dump_points(TA_GlyphHints hints)
             point->oy / 64.0,
             point->x / 64.0,
             point->y / 64.0,
-            (point->flags & TA_FLAG_WEAK_INTERPOLATION) ? 'w' : ' ',
-            (point->flags & TA_FLAG_INFLECTION) ? 'i' : ' ',
-            (point->flags & TA_FLAG_EXTREMA_X) ? '<' : ' ',
-            (point->flags & TA_FLAG_EXTREMA_Y) ? 'v' : ' ',
-            (point->flags & TA_FLAG_ROUND_X) ? '(' : ' ',
-            (point->flags & TA_FLAG_ROUND_Y) ? 'u' : ' '));
+            (point->flags & TA_FLAG_WEAK_INTERPOLATION) ? 'w' : ' '));
   TA_LOG(("\n"));
 }
 
@@ -611,6 +605,9 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
 
       for (point = points; point < point_limit; point++, vec++, tag++)
       {
+        point->in_dir = (FT_Char)TA_DIR_NONE;
+        point->out_dir = (FT_Char)TA_DIR_NONE;
+
         point->fx = (FT_Short)vec->x;
         point->fy = (FT_Short)vec->y;
         point->ox = point->x = FT_MulFix(vec->x, x_scale) + x_delta;
@@ -674,6 +671,7 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
       /* value 20 in `near_limit' is heuristic */
       FT_UInt units_per_em = hints->metrics->scaler.face->units_per_EM;
       FT_Int near_limit = 20 * units_per_em / 2048;
+      FT_Int near_limit2 = 2 * near_limit - 1;
 
       TA_Point* contour;
       TA_Point* contour_limit = hints->contours + hints->num_contours;
@@ -701,8 +699,15 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
           out_x = point->fx - prev->fx;
           out_y = point->fy - prev->fy;
 
-          /* we use Taxicab metrics to measure the vector length */
-          if (TA_ABS(out_x) + TA_ABS(out_y) >= near_limit)
+          /*
+           *  We use Taxicab metrics to measure the vector length.
+           *
+           *  Note that the accumulated distances so far could have the
+           *  opposite direction of the distance measured here.  For this
+           *  reason we use `near_limit2' for the comparison to get a
+           *  non-near point even in the worst case.
+           */
+          if (TA_ABS(out_x) + TA_ABS(out_y) >= near_limit2)
             break;
 
           point = prev;
@@ -716,6 +721,17 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
         /* `in' and `out' vector directions */
 
         curr = first;
+
+        /*
+         *  We abuse the `u' and `v' fields to store index deltas to the
+         *  next and previous non-near point, respectively.
+         *
+         *  To avoid problems with not having non-near points, we point to
+         *  `first' by default as the next non-near point.
+         */
+        curr->u = (FT_Pos)(first - curr);
+        first->v = -curr->u;
+
         out_x = 0;
         out_y = 0;
 
@@ -741,8 +757,6 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
             continue;
           }
 
-          /* we abuse the `u' and `v' fields to store index deltas */
-          /* to the next and previous non-near point, respectively */
           curr->u = (FT_Pos)(next - curr);
           next->v = -curr->u;
 
@@ -758,6 +772,9 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
           }
           next->in_dir = (FT_Char)out_dir;
 
+          curr->u = (FT_Pos)(first - curr);
+          first->v = -curr->u;
+
           out_x = 0;
           out_y = 0;
         }
@@ -864,6 +881,41 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
     }
   }
 
+  /* change some directions at the user's request */
+  /* to make ttfautohint insert one-point segments */
+  /* or remove points from segments */
+  {
+    FONT* font;
+    FT_Int idx;
+    TA_Direction dir;
+    int left_offset;
+    int right_offset;
+
+
+    /* `globals' is not set up while initializing metrics, */
+    /* so exit early in this case */
+    if (!hints->metrics->globals)
+      goto Exit;
+
+    font = hints->metrics->globals->font;
+
+    /* start conditions are set with `TA_control_segment_dir_collect' */
+    while (TA_control_segment_dir_get_next(font, &idx, &dir,
+                                           &left_offset, &right_offset))
+    {
+      TA_Point point = &points[idx];
+
+
+      point->out_dir = dir;
+      if (dir == TA_DIR_NONE)
+        point->flags |= TA_FLAG_WEAK_INTERPOLATION;
+      else
+        point->flags &= ~TA_FLAG_WEAK_INTERPOLATION;
+      point->left_offset = (FT_Short)left_offset;
+      point->right_offset = (FT_Short)right_offset;
+    }
+  }
+
 Exit:
   return error;
 }
@@ -1020,8 +1072,7 @@ ta_glyph_hints_align_strong_points(TA_GlyphHints hints,
       /* if this point is candidate to weak interpolation, we */
       /* interpolate it after all strong points have been processed */
 
-      if ((point->flags & TA_FLAG_WEAK_INTERPOLATION)
-          && !(point->flags & TA_FLAG_INFLECTION))
+      if ((point->flags & TA_FLAG_WEAK_INTERPOLATION))
         continue;
 
       if (dim == TA_DIMENSION_VERT)
