@@ -40,7 +40,8 @@
 
 void
 ta_latin_metrics_init_widths(TA_LatinMetrics metrics,
-                             FT_Face face)
+                             FT_Face face,
+                             FT_Bool use_cmap)
 {
   /* scan the array of segments in each direction */
   TA_GlyphHintsRec hints[1];
@@ -71,6 +72,9 @@ ta_latin_metrics_init_widths(TA_LatinMetrics metrics,
     FT_UInt32 standard_char;
 
 
+    if (!use_cmap)
+      goto Exit;
+
     /*
      * We check more than a single standard character to catch features
      * like `c2sc' (small caps from caps) that don't contain lowercase
@@ -192,12 +196,29 @@ ta_latin_metrics_init_widths(TA_LatinMetrics metrics,
   Exit:
     for (dim = 0; dim < TA_DIMENSION_MAX; dim++)
     {
+      FONT* font = metrics->root.globals->font;
       TA_LatinAxis axis = &metrics->axis[dim];
       FT_Pos stdw;
 
 
-      stdw = (axis->width_count > 0) ? axis->widths[0].org
-                                     : TA_LATIN_CONSTANT(metrics, 50);
+      if (!axis->width_count)
+      {
+        /* if we have no standard characters, */
+        /* use `fallback-stem-width', if available, */
+        /* or a default width (value 50 is heuristic) */
+        stdw = (dim == TA_DIMENSION_VERT && font->fallback_stem_width)
+                 ? (FT_Pos)font->fallback_stem_width
+                 : TA_LATIN_CONSTANT(metrics, 50);
+
+        /* set one width value if we do hinting */
+        if (style_class->style != TA_STYLE_NONE_DFLT)
+        {
+          axis->width_count++;
+          axis->widths[0].org = stdw;
+        }
+      }
+
+      stdw = axis->widths[0].org;
 
       /* let's try 20% of the smallest width */
       axis->edge_distance_threshold = stdw / 5;
@@ -515,6 +536,13 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
             FT_Int last;
             FT_Bool hit;
 
+            /* we intentionally declare these two variables */
+            /* outside of the loop since various compilers emit */
+            /* incorrect warning messages otherwise, talking about */
+            /* `possibly uninitialized variables' */
+            FT_Int p_first = 0; /* make compiler happy */
+            FT_Int p_last  = 0;
+
             FT_Bool left2right;
 
 
@@ -546,8 +574,6 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
             {
               FT_Bool l2r;
               FT_Pos d;
-              FT_Int p_first;
-              FT_Int p_last;
 
 
               if (!hit)
@@ -882,10 +908,15 @@ ta_latin_metrics_init(TA_LatinMetrics metrics,
 
   if (!FT_Select_Charmap(face, FT_ENCODING_UNICODE))
   {
-    ta_latin_metrics_init_widths(metrics, face);
+    ta_latin_metrics_init_widths(metrics, face, 1);
     ta_latin_metrics_init_blues(metrics, face);
     ta_latin_metrics_check_digits(metrics, face);
   }
+  else
+  {
+    /* we only have a symbol font encoding */
+    ta_latin_metrics_init_widths(metrics, face, 0);
+  }
 
   FT_Set_Charmap(face, oldmap);
   return FT_Err_Ok;
@@ -1331,17 +1362,36 @@ ta_latin_hints_compute_segments(TA_GlyphHints hints,
         /* this is the start of a new segment! */
         segment_dir = (TA_Direction)point->out_dir;
 
-        /* clear all segment fields */
         error = ta_axis_hints_new_segment(axis, &segment);
         if (error)
           goto Exit;
 
+        /* clear all segment fields */
         segment[0] = seg0;
+
         segment->dir = (FT_Char)segment_dir;
         min_pos = max_pos = point->u;
         segment->first = point;
         segment->last = point;
+
         on_edge = 1;
+
+        if (point->out_dir != point->next->in_dir)
+        {
+          /* we have a one-point segment */
+          segment->pos = (FT_Short)min_pos;
+
+          if (point->flags & TA_FLAG_CONTROL)
+            segment->flags |= TA_EDGE_ROUND;
+
+          /* artificially extend the horizontal size if requested */
+          segment->min_coord = (FT_Short)point->v + point->left_offset;
+          segment->max_coord = (FT_Short)point->v + point->right_offset;
+          segment->height = 0;
+
+          on_edge = 0;
+          segment = NULL;
+        }
       }
 
       point = point->next;
@@ -1365,9 +1415,6 @@ ta_latin_hints_compute_segments(TA_GlyphHints hints,
       FT_Pos last_v = last->v;
 
 
-      if (first == last)
-        continue;
-
       if (first_v < last_v)
       {
         TA_Point p;
@@ -1445,10 +1492,7 @@ ta_latin_hints_link_segments(TA_GlyphHints hints,
   /* now compare each segment to the others */
   for (seg1 = segments; seg1 < segment_limit; seg1++)
   {
-    /* the fake segments are introduced to hint the metrics -- */
-    /* we must never link them to anything */
-    if (seg1->dir != axis->major_dir
-        || seg1->first == seg1->last)
+    if (seg1->dir != axis->major_dir)
       continue;
 
     /* search for stems having opposite directions, */
@@ -1476,6 +1520,14 @@ ta_latin_hints_link_segments(TA_GlyphHints hints,
         /* compute maximum coordinate difference of the two segments */
         /* (this is, how much they overlap) */
         len = max - min;
+
+        /* for one-point segments, `len' is zero if there is an overlap */
+        /* (and negative otherwise); we have to correct this */
+        if (len == 0
+            && (seg1->min_coord == seg1->max_coord
+                || seg2->min_coord == seg2->max_coord))
+          len = len_threshold;
+
         if (len >= len_threshold)
         {
           /*
@@ -2915,7 +2967,7 @@ ta_latin_hints_apply(TA_GlyphHints hints,
   FT_Error error;
   int dim;
 
-  TA_LatinAxis  axis;
+  TA_LatinAxis axis;
 
 
   error = ta_glyph_hints_reload(hints, outline);
