@@ -1,7 +1,7 @@
 /* tatime.c */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -14,24 +14,11 @@
 
 
 #include <time.h>
+#include <stdint.h>
 
 #include "ta.h"
 
-/* we need an unsigned 64bit data type */
-
-/* make `stdint.h' define `uintXX_t' for C++ */
-#undef __STDC_LIMIT_MACROS
-#define __STDC_LIMIT_MACROS
-
-#if HAVE_STDINT_H
-#  include <stdint.h>
-#endif
-
-#if defined UINT64_MAX || defined uint64_t
 typedef uint64_t TA_ULongLong;
-#else
-#  error "No unsigned 64bit wide data type found."
-#endif
 
 
 void
