@@ -26,21 +26,13 @@
 #define COMPARE(str) \
           (len == (sizeof (str) - 1) \
            && !strncmp(start, str, sizeof (str) - 1))
-#define DUMPVAL(str, arg) \
-          fprintf(stderr, "%33s = %ld\n", \
-                          (str), \
-                          (FT_Long)(arg))
-#define DUMPSTR(str, arg) \
-          fprintf(stderr, "%33s = %s\n", \
-                          (str), \
-                          (arg))
 
 
 void
 TA_sfnt_set_properties(SFNT* sfnt,
                        FONT* font)
 {
-  TA_FaceGlobals  globals = (TA_FaceGlobals)sfnt->face->autohint.data;
+  TA_FaceGlobals globals = (TA_FaceGlobals)sfnt->face->autohint.data;
 
 
   globals->increase_x_height = font->increase_x_height;
@@ -54,16 +46,26 @@ TTF_autohint(const char* options,
   va_list ap;
 
   FONT* font;
-  FT_Error error;
   FT_Long i;
 
+  TA_Error error;
+  char* error_string = NULL;
+  unsigned int errlinenum = 0;
+  char* errline = NULL;
+  char* errpos = NULL;
+  FT_Bool free_errline = 0;
+  FT_Bool free_error_string = 0;
+
   FILE* in_file = NULL;
   FILE* out_file = NULL;
+  FILE* control_file = NULL;
 
   const char* in_buf = NULL;
   size_t in_len = 0;
   char** out_bufp = NULL;
   size_t* out_lenp = NULL;
+  const char* control_buf = NULL;
+  size_t control_len = 0;
 
   const unsigned char** error_stringp = NULL;
 
@@ -75,18 +77,22 @@ TTF_autohint(const char* options,
   const char* x_height_snapping_exceptions_string = NULL;
   number_range* x_height_snapping_exceptions = NULL;
 
+  FT_Long fallback_stem_width = 0;
+
   FT_Bool gray_strong_stem_width = 0;
   FT_Bool gdi_cleartype_strong_stem_width = 1;
   FT_Bool dw_cleartype_strong_stem_width = 0;
 
-  TA_Progress_Func progress;
-  void* progress_data;
-  TA_Info_Func info;
-  void* info_data;
+  TA_Progress_Func progress = NULL;
+  void* progress_data = NULL;
+  TA_Error_Func err = NULL;
+  void* err_data = NULL;
+  TA_Info_Func info = NULL;
+  void* info_data = NULL;
 
   FT_Bool windows_compatibility = 0;
   FT_Bool ignore_restrictions = 0;
-  FT_Bool pre_hinting = 0;
+  FT_Bool adjust_subglyphs = 0;
   FT_Bool hint_composites = 0;
   FT_Bool symbol = 0;
 
@@ -96,22 +102,11 @@ TTF_autohint(const char* options,
   TA_Script default_script = TA_SCRIPT_LATN;
 
   FT_Bool dehint = 0;
-
   FT_Bool debug = 0;
+  FT_Bool TTFA_info = 0;
 
   const char* op;
 
-#undef SCRIPT
-#define SCRIPT(s, S, d, h, sc1, sc2, sc3) #s,
-
-  const char* script_names[] =
-  {
-
-#include <ttfautohint-scripts.h>
-
-  };
-
-
   if (!options || !*options)
   {
     error = FT_Err_Invalid_Argument;
@@ -148,18 +143,42 @@ TTF_autohint(const char* options,
     /* the `COMPARE' macro uses `len' and `start' */
 
     /* handle options -- don't forget to update parameter dump below! */
-    if (COMPARE("debug"))
+    if (COMPARE("adjust-subglyphs"))
+      adjust_subglyphs = (FT_Bool)va_arg(ap, FT_Int);
+    else if (COMPARE("debug"))
       debug = (FT_Bool)va_arg(ap, FT_Int);
     else if (COMPARE("default-script"))
       default_script_string = va_arg(ap, const char*);
     else if (COMPARE("dehint"))
       dehint = (FT_Bool)va_arg(ap, FT_Int);
+    else if (COMPARE("control-buffer"))
+    {
+      control_file = NULL;
+      control_buf = va_arg(ap, const char*);
+    }
+    else if (COMPARE("control-buffer-len"))
+    {
+      control_file = NULL;
+      control_len = va_arg(ap, size_t);
+    }
+    else if (COMPARE("control-file"))
+    {
+      control_file = va_arg(ap, FILE*);
+      control_buf = NULL;
+      control_len = 0;
+    }
     else if (COMPARE("dw-cleartype-strong-stem-width"))
       dw_cleartype_strong_stem_width = (FT_Bool)va_arg(ap, FT_Int);
+    else if (COMPARE("error-callback"))
+      err = va_arg(ap, TA_Error_Func);
+    else if (COMPARE("error-callback-data"))
+      err_data = va_arg(ap, void*);
     else if (COMPARE("error-string"))
       error_stringp = va_arg(ap, const unsigned char**);
     else if (COMPARE("fallback-script"))
       fallback_script_string = va_arg(ap, const char*);
+    else if (COMPARE("fallback-stem-width"))
+      fallback_stem_width = (FT_Long)va_arg(ap, FT_UInt);
     else if (COMPARE("gdi-cleartype-strong-stem-width"))
       gdi_cleartype_strong_stem_width = (FT_Bool)va_arg(ap, FT_Int);
     else if (COMPARE("gray-strong-stem-width"))
@@ -213,13 +232,15 @@ TTF_autohint(const char* options,
       out_lenp = NULL;
     }
     else if (COMPARE("pre-hinting"))
-      pre_hinting = (FT_Bool)va_arg(ap, FT_Int);
+      adjust_subglyphs = (FT_Bool)va_arg(ap, FT_Int);
     else if (COMPARE("progress-callback"))
       progress = va_arg(ap, TA_Progress_Func);
     else if (COMPARE("progress-callback-data"))
       progress_data = va_arg(ap, void*);
     else if (COMPARE("symbol"))
       symbol = (FT_Bool)va_arg(ap, FT_Int);
+    else if (COMPARE("TTFA-info"))
+      TTFA_info = (FT_Bool)va_arg(ap, FT_Int);
     else if (COMPARE("windows-compatibility"))
       windows_compatibility = (FT_Bool)va_arg(ap, FT_Int);
     else if (COMPARE("x-height-snapping-exceptions"))
@@ -349,7 +370,12 @@ TTF_autohint(const char* options,
                                      0x7FFF);
     if (*s)
     {
-      error = FT_Err_Invalid_Argument;
+      /* we map numberset.h's error codes to values starting with 0x100 */
+      error = 0x100 - (FT_Error)x_height_snapping_exceptions;
+      errlinenum = 0;
+      errline = (char*)x_height_snapping_exceptions_string;
+      errpos = (char*)s;
+
       goto Err1;
     }
   }
@@ -359,6 +385,7 @@ TTF_autohint(const char* options,
   font->hinting_limit = (FT_UInt)hinting_limit;
   font->increase_x_height = increase_x_height;
   font->x_height_snapping_exceptions = x_height_snapping_exceptions;
+  font->fallback_stem_width = (FT_UInt)fallback_stem_width;
 
   font->gray_strong_stem_width = gray_strong_stem_width;
   font->gdi_cleartype_strong_stem_width = gdi_cleartype_strong_stem_width;
@@ -366,7 +393,7 @@ TTF_autohint(const char* options,
 
   font->windows_compatibility = windows_compatibility;
   font->ignore_restrictions = ignore_restrictions;
-  font->pre_hinting = pre_hinting;
+  font->adjust_subglyphs = adjust_subglyphs;
   font->hint_composites = hint_composites;
   font->fallback_style = fallback_style;
   font->default_script = default_script;
@@ -380,63 +407,11 @@ No_check:
 
   font->debug = debug;
   font->dehint = dehint;
+  font->TTFA_info = TTFA_info;
 
   font->gasp_idx = MISSING;
 
-  /* dump parameters */
-  if (debug)
-  {
-    fprintf(stderr, "TTF_autohint parameters\n"
-                    "=======================\n"
-                    "\n");
-
-    if (dehint)
-      DUMPVAL("dehint",
-              font->dehint);
-    else
-    {
-      char *s;
-
-
-      DUMPSTR("default-script",
-              script_names[font->default_script]);
-      DUMPVAL("dw-cleartype-strong-stem-width",
-              font->dw_cleartype_strong_stem_width);
-      DUMPSTR("fallback-script",
-              script_names[ta_style_classes[font->fallback_style]->script]);
-      DUMPVAL("gdi-cleartype-strong-stem-width",
-              font->gdi_cleartype_strong_stem_width);
-      DUMPVAL("gray-strong-stem-width",
-              font->gray_strong_stem_width);
-      DUMPVAL("hinting-limit",
-              font->hinting_limit);
-      DUMPVAL("hinting-range-max",
-              font->hinting_range_max);
-      DUMPVAL("hinting-range-min",
-              font->hinting_range_min);
-      DUMPVAL("hint-composites",
-              font->hint_composites);
-      DUMPVAL("ignore-restrictions",
-              font->ignore_restrictions);
-      DUMPVAL("increase-x-height",
-              font->increase_x_height);
-      DUMPVAL("pre-hinting",
-              font->pre_hinting);
-      DUMPVAL("symbol",
-              font->symbol);
-      DUMPVAL("windows-compatibility",
-              font->windows_compatibility);
-
-      s = number_set_show(font->x_height_snapping_exceptions,
-                          TA_PROP_INCREASE_X_HEIGHT_MIN, 0x7FFF);
-      DUMPSTR("x-height-snapping-exceptions", s);
-      free(s);
-    }
-
-    fprintf(stderr, "\n");
-  }
-
-  /* now start with processing the data */
+  /* start with processing the data */
 
   if (in_file)
   {
@@ -456,6 +431,18 @@ No_check:
     font->in_len = in_len;
   }
 
+  if (control_file)
+  {
+    error = TA_control_file_read(font, control_file);
+    if (error)
+      goto Err;
+  }
+  else if (control_buf)
+  {
+    font->control_buf = (char*)control_buf;
+    font->control_len = control_len;
+  }
+
   error = TA_font_init(font);
   if (error)
     goto Err;
@@ -466,7 +453,8 @@ No_check:
     _ta_debug_global = 1;
   }
 
-  /* we do some loops over all subfonts */
+  /* we do some loops over all subfonts -- */
+  /* to process options early, just start with loading all of them */
   for (i = 0; i < font->num_sfnts; i++)
   {
     SFNT* sfnt = &font->sfnts[i];
@@ -487,6 +475,45 @@ No_check:
 
     if (error)
       goto Err;
+  }
+
+  /* process control instructions */
+  error = TA_control_parse_buffer(font,
+                                  &error_string,
+                                  &errlinenum, &errline, &errpos);
+  if (error)
+  {
+    free_errline = 1;
+    free_error_string = 1;
+    goto Err;
+  }
+
+  /* now we are able to dump all parameters */
+  if (debug)
+  {
+    char* s;
+
+
+    s = TA_font_dump_parameters(font, 1);
+    if (!s)
+    {
+      error = FT_Err_Out_Of_Memory;
+      goto Err;
+    }
+
+    fprintf(stderr, "%s", s);
+    free(s);
+  }
+
+  error = TA_control_build_tree(font);
+  if (error)
+    goto Err;
+
+  /* loop again over subfonts and continue processing */
+  for (i = 0; i < font->num_sfnts; i++)
+  {
+    SFNT* sfnt = &font->sfnts[i];
+
 
     error = TA_sfnt_split_into_SFNT_tables(sfnt, font);
     if (error)
@@ -515,7 +542,7 @@ No_check:
     }
     else
     {
-      if (font->pre_hinting)
+      if (font->adjust_subglyphs)
         error = TA_sfnt_create_glyf_data(sfnt, font);
       else
         error = TA_sfnt_split_glyf_table(sfnt, font);
@@ -539,8 +566,7 @@ No_check:
       SFNT* sfnt = &font->sfnts[i];
 
 
-      if (TA_sfnt_adjust_master_coverage(sfnt, font))
-        break;
+      TA_sfnt_adjust_coverage(sfnt, font);
     }
   }
 
@@ -611,7 +637,7 @@ No_check:
     {
       /* we add one glyph for composites */
       if (sfnt->max_components
-          && !font->pre_hinting
+          && !font->adjust_subglyphs
           && font->hint_composites)
       {
         error = TA_sfnt_update_hmtx_table(sfnt, font);
@@ -657,12 +683,31 @@ No_check:
   error = TA_Err_Ok;
 
 Err:
-  TA_font_unload(font, in_buf, out_bufp);
+  TA_control_free(font->control);
+  TA_control_free_tree(font);
+  TA_font_unload(font, in_buf, out_bufp, control_buf);
 
 Err1:
+  if (!error_string)
+    error_string = (char*)TA_get_error_message(error);
+
+  /* this must be a static value */
   if (error_stringp)
     *error_stringp = (const unsigned char*)TA_get_error_message(error);
 
+  if (err)
+    err(error,
+        error_string,
+        errlinenum,
+        errline,
+        errpos,
+        err_data);
+
+  if (free_errline)
+    free(errline);
+  if (free_error_string)
+    free(error_string);
+
   return error;
 }
 
