@@ -110,6 +110,45 @@ typedef int
  */
 
 
+/*
+ * Callback: `TA_Error_Func`
+ * -------------------------
+ *
+ * A callback function to get error information.
+ *
+ * *error* is the value `TTF_autohint` returns.  See file
+ * `ttfautohint-errors.h` for a list.  Error codes not in this list are
+ * directly taken from FreeType; see the FreeType header file `fterrdef.h`
+ * for more.
+ *
+ * *error_string*, if non-NULL, is a pointer to an error message that
+ * represents *error*.
+ *
+ * The next three parameters help identify the origin of text string parsing
+ * errors.  *linenum*, if non-zero, contains the line number.  *line*, if
+ * non-NULL, is a pointer to the input line that can't be processed.
+ * *errpos*, if non-NULL, holds a pointer to the position in *line* where
+ * the problem occurs.
+ *
+ * *error_data* is a void pointer to user-supplied data.
+ *
+ * ```C
+ */
+
+typedef void
+(*TA_Error_Func)(TA_Error error,
+                 const char* error_string,
+                 unsigned int linenum,
+                 const char* line,
+                 const char* errpos,
+                 void* error_data);
+
+/*
+ * ```
+ *
+ */
+
+
 /*
  * Callback: `TA_Info_Func`
  * ------------------------
@@ -210,6 +249,88 @@ typedef int
  * :   A pointer of type `size_t*` to a value giving the length of the
  *     output buffer.  Needs `out-buffer`.
  *
+ * `control-file`
+ * :   A pointer of type `FILE*` to the data stream of control instructions.
+ *     Mutually exclusive with `control-buffer`.
+ *
+ *     An entry in a control instructions file or buffer has one of the
+ *     following syntax forms:
+ *
+ *     > *\[* font-idx *\]*\ \ glyph-id\ \ *`l`|`r`* points\ \ *\[* *`(`* left-offset *`,`* right-offset *`)`* *\]*\
+ *     > *\[* font-idx *\]*\ \ glyph-id\ \ *`n`* points\
+ *     > *\[* font-idx *\]*\ \ glyph-id\ \ *`p`* points\ \ *\[* *`x`* x-shift *\]*\ \ *\[* *`y`* y-shift *\]*\ \ *`@`* ppems
+ *
+ *     *font-idx* gives the index of the font in a TrueType Collection.  If
+ *     missing, it is set to zero.  For normal TrueType fonts, only value
+ *     zero is valid.  If starting with `0x` the number is interpreted as
+ *     hexadecimal.  If starting with `0` it gets interpreted as an octal
+ *     value, and as a decimal value otherwise.
+ *
+ *     *glyph-id* is a glyph's name as listed in the `post` SFNT table or a
+ *     glyph index.  A glyph name consists of characters from the set
+ *     '`A-Za-z0-9._`' only and does not start with a digit or period, with
+ *     the exceptions of the names '`.notdef`' and '`.null`'.  A glyph index
+ *     can be specified in decimal, octal, or hexadecimal format, the latter
+ *     two indicated by the prefixes `0` and `0x`, respectively.
+ *
+ *     The mutually exclusive parameters '`l`' and '`r`' indicate that the
+ *     following points have left or right 'out' direction, respectively,
+ *     overriding ttfautohint's algorithm for setting point directions.  The
+ *     'out direction' of a point is the direction of the outline *leaving*
+ *     the point (or passing the control point).  If the specified direction
+ *     is identical to what ttfautohint computes, nothing special happens.
+ *     Otherwise, a one-point segment with the specified direction gets
+ *     created.  By default, its length is zero.  Setting *left-offset* and
+ *     *right-offset*, you can change the segment's horizontal start and end
+ *     position relative to the point position.  *left-offset* and
+ *     *right-offset* are integers measured in font units.
+ *
+ *     Parameter '`n`' sets the 'out' direction of the following points to
+ *     'no direction'.  If the specified direction is identical to what
+ *     ttfautohint computes, nothing special happens.  Otherwise,
+ *     ttfautohint no longer considers those points as part of horizontal
+ *     segments, thus treating them as 'weak' points.
+ *
+ *     Modifying or adding segments don't directly modify the outlines; it
+ *     only influences the hinting process.
+ *
+ *     Parameter '`p`' makes ttfautohint apply delta exceptions for the
+ *     given points, shifting the points by the given values.  Note that
+ *     those delta exceptions are applied *after* the final `IUP`
+ *     instructions in the bytecode; as a consequence, they are (partially)
+ *     ignored by rasterizers if in ClearType mode.
+ *
+ *     Both *points* and *ppems* are number ranges, similar to the
+ *     `x-height-snapping-exceptions` syntax.
+ *
+ *     *x-shift* and *y-shift* represent real numbers that get rounded to
+ *     multiples of 1/8 pixels.  The entries for '`x`' and '`y`' are
+ *     optional; if missing, the corresponding value is set to zero.
+ *
+ *     Values for *x-shift* and *y-shift* must be in the range [-1.0;1.0].
+ *     Values for *ppems* must be in the range [6;53].  Values for *points*
+ *     are limited by the number of points in the glyph.
+ *
+ *     Control instruction entries can be either separated with newlines or
+ *     with character '`;`'.  Additionally, a line can be continued on the
+ *     next line by ending it with backslash character ('`\`').  A backslash
+ *     followed by a newline gets treated similar to a whitespace character.
+ *
+ *     A comment starts with character '`#`'; the rest of the line is
+ *     ignored.  An empty line is ignored also.
+ *
+ *     Note that only character '`.`' is recognized as a decimal point, and
+ *     a thousands separator is not accepted.
+ *
+ * `control-buffer`
+ * :   A pointer of type `const char*` to a buffer that contains control
+ *     instructions.  Needs `control-buffer-len`.  Mutually exclusive with
+ *     `control-file`.
+ *
+ * `control-buffer-len`
+ * :   A value of type `size_t`, giving the length of the control
+ *     instructions buffer.  Needs `control-buffer`.
+ *
  *
  * ### Messages and Callbacks
  *
@@ -228,6 +349,18 @@ typedef int
  *     that verbally describes the error code.  You must not change the
  *     returned value.
  *
+ * `error-callback`
+ * :   A pointer of type [`TA_Error_Func`](#callback-ta_error_func),
+ *     specifying a callback function for error messages.  This function
+ *     gets called right before `TTF_autohint` exits.  If this field is not
+ *     set or set to NULL, no error callback function is used.
+ *
+ *     Use it as a more sophisticated alternative to `error-string`.
+ *
+ * `error-callback-data`
+ * :   A point of type `void*` to user data that is passed to the error
+ *     callback function.
+ *
  * `info-callback`
  * :   A pointer of type [`TA_Info_Func`](#callback-ta_info_func),
  *     specifying a callback function for manipulating the `name` table.
@@ -273,13 +406,16 @@ typedef int
  *     results.  However, this depends on the processed font and must be
  *     checked by inspection.
  *
- * `pre-hinting`
+ * `adjust-subglyphs`
  * :   An integer (1\ for 'on' and 0\ for 'off', which is the default) to
- *     specify whether native TrueType hinting shall be applied to all
- *     glyphs before passing them to the (internal) autohinter.  The used
- *     resolution is the em-size in font units; for most fonts this is
- *     2048ppem.  Use this if the hints move or scale subglyphs
- *     independently of the output resolution.
+ *     specify whether native TrueType hinting of the *input font* shall be
+ *     applied to all glyphs before passing them to the (internal)
+ *     autohinter.  The used resolution is the em-size in font units; for
+ *     most fonts this is 2048ppem.  Use this only if the old hints move or
+ *     scale subglyphs independently of the output resolution, for example
+ *     some exotic CJK fonts.
+ *
+ *     `pre-hinting` is a deprecated alias name for this option.
  *
  *
  * ### Hinting Algorithms
@@ -305,8 +441,8 @@ typedef int
  *     also.
  *
  * `increase-x-height`
- * :   An integer.  For PPEM values in the range 6\ <= PPEM <=\
- *     `increase-x-height`, round up the font's x\ height much more often
+ * :   An integer.  For PPEM values in the range 6\ <= PPEM
+ *     <= `increase-x-height`, round up the font's x\ height much more often
  *     than normally (to use the terminology of TrueType's 'Super Round'
  *     bytecode instruction, the threshold gets increased from 5/8px to
  *     13/16px).  If it is set to\ 0, this feature is switched off.  If this
@@ -316,15 +452,15 @@ typedef int
  * `x-height-snapping-exceptions`
  * :   A pointer of type `const char*` to a null-terminated string that
  *     gives a list of comma separated PPEM values or value ranges at which
- *     no x-height snapping shall be applied.  A value range has the form
- *     *value1*`-`*value2*, meaning *value1* <= PPEM <= *value2*.  *value1*
- *     or *value2* (or both) can be missing; a missing value is replaced by
- *     the beginning or end of the whole interval of valid PPEM values,
- *     respectively.  Whitespace is not significant; superfluous commas are
- *     ignored, and ranges must be specified in increasing order.  For
- *     example, the string `"3, 5-7, 9-"` means the values 3, 5, 6, 7, 9,
- *     10, 11, 12, etc.  Consequently, if the supplied argument is `"-"`, no
- *     x-height snapping takes place at all.  The default is the empty
+ *     no x\ height snapping shall be applied.  A value range has the form
+ *     *value*~1~`-`*value*~2~, meaning *value*~1~ <= PPEM <= *value*~2~.
+ *     *value*~1~ or *value*~2~ (or both) can be missing; a missing value is
+ *     replaced by the beginning or end of the whole interval of valid PPEM
+ *     values, respectively.  Whitespace is not significant; superfluous
+ *     commas are ignored, and ranges must be specified in increasing order.
+ *     For example, the string `"3, 5-7, 9-"` means the values 3, 5, 6, 7,
+ *     9, 10, 11, 12, etc.  Consequently, if the supplied argument is `"-"`,
+ *     no x\ height snapping takes place at all.  The default is the empty
  *     string (`""`), meaning no snapping exceptions.
  *
  * `windows-compatibility`
@@ -353,12 +489,25 @@ typedef int
  *     `ttfautohint-scripts.h`.
  *
  * `symbol`
- * :   Set this integer to\ 1 if you want to process a font that lacks the
- *     characters of a supported script, for example, a symbol font.
- *     ttfautohint then uses default values for the standard stem width and
- *     height instead of deriving these values from a script's standard
- *     characters (for the latin script, one of them is character 'o').  The
- *     default value is\ 0.
+ * :   Set this integer to\ 1 if you want to process a font that ttfautohint
+ *     would refuse otherwise because it can't find a single standard
+ *     character for any of the supported scripts.  ttfautohint then uses a
+ *     default (hinting) value for the standard stem width instead of
+ *     deriving it from a script's set of standard characters (for the latin
+ *     script, one of them is character 'o').  The default value of this
+ *     option is\ 0.
+ *
+ * `fallback-stem-width`
+ * :   Set the horizontal stem width (hinting) value for all scripts that
+ *     lack proper standard characters.  The value is given in font units
+ *     and must be a positive integer.  If not set, or the value is zero,
+ *     ttfautohint uses a hard-coded default (50\ units at 2048 units per
+ *     EM, and linearly scaled for other UPEM values, for example 24\ units
+ *     at 1000 UPEM).
+ *
+ *     For symbol fonts (i.e., option `symbol` is given),
+ *     `fallback-stem-width` has an effect only if `fallback-script` is set
+ *     also.
  *
  *
  * ### Miscellaneous
@@ -371,6 +520,26 @@ typedef int
  *     value\ 1 to make ttfautohint handle the font.  The default value
  *     is\ 0.
  *
+ * `TTFA-info`
+ * :   If set to\ 1, ttfautohint creates an SFNT table called `TTFA` and
+ *     fills it with information on the parameters used while calling
+ *     `TTF_autohint`.  The format of the output data resembles the
+ *     information at the very beginning of the dump emitted by option
+ *     `debug`.  The default value is\ 0.
+ *
+ *     Main use of this option is for font editing purposes.  For example,
+ *     after a font editor has added some glyphs, a front-end to
+ *     `TTF_autohint` can parse `TTFA` and feed the parameters into another
+ *     call of `TTF_autohint`.  The new glyphs are then hinted while hints
+ *     of the old glyphs stay unchanged.
+ *
+ *     If this option is not set, and the font to be processed contains a
+ *     `TTFA` table, it gets removed.
+ *
+ *     Note that such a `TTFA` table gets ignored by all font rendering
+ *     engines.  In TrueType Collections, the `TTFA` table is added to the
+ *     first subfont.
+ *
  * `dehint`
  * :   If set to\ 1, remove all hints from the font.  All other hinting
  *     options are ignored.
